//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

/*!
 * Category class to download thumbnail of file.
 */
#import "PCAPISDKCommonHeader.h"
@class ImageDownloadView;
@interface PCFile (GridCell)

/*!
 * @discussion call this method to set thumbnail image to an image view. This method checks for cached image, if not found it downloads image using the content token stored in PCFile. If this fails a default image is set for the image view.
 * @param imageView the imageView for which thumnail needs to be set.
 * @param completion completion block gets called once imageView is set with an image. didSetImage indicates if an image has been set to the image view or not. By the time image is downloaded and if imageView represents a different resource now, downloaded image is not set to the image. In that case didSetImage is returned NO.
 */
- (void)setThumbnailForImageView:(ImageDownloadView *)imageView withCompletionHandler:(void (^)(BOOL didSetImage))completion;

/*!
 * @discussion call this method to set a full resolution image to an image view. This method checks for cached image, if not found it downloads image using the image url stored in PCFile. If this fails a default image is set for the image view.
 * @param imageView the imageView for which full resolution image needs to be set.
 * @param completion completion block gets called once imageView is set with an image. didSetImage indicates if an image has been set to the image view or not. By the time image is downloaded and if imageView represents a different resource now, downloaded image is not set to the image. In that case didSetImage is returned NO.
 */
- (void)setFullResolutionImageView:(ImageDownloadView *)imageView withProgressHandler:(void (^)(float progress))progressCompletion andCompletionHandler:(void (^)(BOOL didSetImage))completion;

@end
