//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import "CNContact+Additions.h"

@implementation CNContact (Additions)

- (PCContact *)getPCContact {
    
    NSMutableDictionary *dictionary = [NSMutableDictionary new];
    if (self.givenName) {
        [dictionary setObject:self.givenName forKey:@"firstname"];
    }
    if (self.familyName) {
        [dictionary setObject:self.familyName forKey:@"lastname"];
    }
    if (self.namePrefix) {
        [dictionary setObject:self.namePrefix forKey:@"nameprefix"];
    }
    if (self.nameSuffix) {
        [dictionary setObject:self.nameSuffix forKey:@"namesuffix"];
    }
    if (self.nickname) {
        [dictionary setObject:self.nickname forKey:@"nickname"];
    }
    if (self.jobTitle) {
        [dictionary setObject:self.jobTitle forKey:@"jobtitle"];
    }
    if (self.organizationName) {
        [dictionary setObject:self.organizationName forKey:@"orgname"];
    }
    if (self.note) {
        [dictionary setObject:self.note forKey:@"note"];
    }
    if (self.middleName) {
        [dictionary setObject:self.middleName forKey:@"middlename"];
    }
    if (self.departmentName) {
        [dictionary setObject:self.departmentName forKey:@"orgunit"];
    }
    if (self.birthday) {
        [dictionary setObject:[self convertDateToServerSupportedFormat:self.birthday.date] forKey:@"birthday"];
    }
    
    if (self.phoneNumbers) {
        NSMutableArray *telephoneArray = [NSMutableArray new];
        for (CNLabeledValue *telephone in self.phoneNumbers) {
            NSMutableDictionary *telDict = [NSMutableDictionary new];
            if (telephone.label) {
               [telDict setObject:[self fetchServerSupportedString:telephone.label] forKey:@"type"];
            }
            if (telephone.value) {
                CNPhoneNumber *number = telephone.value;
                 [telDict setObject:number.stringValue forKey:@"number"];
            }
            [telDict setObject:[NSNumber numberWithInteger:0] forKey:@"indx"];
            [telDict setObject:[NSNumber numberWithInteger:0] forKey:@"preference"];
            [telephoneArray addObject:telDict];
            
        }
        [dictionary setObject:telephoneArray forKey:@"tel"];

    }
    if (self.emailAddresses) {
        
        NSMutableArray *emailArray = [NSMutableArray new];
        
        for (CNLabeledValue *email in self.emailAddresses) {
            
            NSMutableDictionary *emailDict = [NSMutableDictionary new];
            if (email.label) {
                [emailDict setObject:[self fetchServerSupportedString:email.label] forKey:@"type"];
            }
            if (email.value) {
                [emailDict setObject:email.value forKey:@"address"];

            }
            [emailDict setObject:[NSNumber numberWithInteger:0] forKey:@"indx"];
            [emailDict setObject:[NSNumber numberWithInteger:0] forKey:@"preference"];
            [emailArray addObject:emailDict];
            
        }
        [dictionary setObject:emailArray forKey:@"email"];
        
    }
    
    if (self.instantMessageAddresses) {
        
        NSMutableArray *IMArray = [NSMutableArray new];
        
        for (CNLabeledValue *im in self.instantMessageAddresses) {
            
            NSMutableDictionary *imDict = [NSMutableDictionary new];
            CNInstantMessageAddress *imAddress = im.value;
            if (imAddress.service) {
                [imDict setObject:[self fetchServerSupportedString:imAddress.service] forKey:@"type"];
            }
            if (imAddress.username) {
                [imDict setObject:imAddress.username forKey:@"address"];
            }
            [imDict setObject:[NSNumber numberWithInteger:0] forKey:@"preference"];
            [IMArray addObject:imDict];
            
        }
        
        [dictionary setObject:IMArray forKey:@"im"];
        
    }
    
    if (self.postalAddresses) {
        
        NSMutableArray *addressArray = [NSMutableArray new];
        
        for (CNLabeledValue *address in self.postalAddresses) {
            
            NSMutableDictionary *addressDict = [NSMutableDictionary new];
            CNMutablePostalAddress *postalAddress = address.value;
            if (address.label) {
               [addressDict setObject:[self fetchServerSupportedString:address.label] forKey:@"type"];
            }
            if (postalAddress.city) {
                [addressDict setObject:postalAddress.city forKey:@"city"];
            }
            if (postalAddress.state) {
                [addressDict setObject:postalAddress.state forKey:@"state"];
            }
            if (postalAddress.postalCode) {
                [addressDict setObject:postalAddress.postalCode forKey:@"zipcode"];
            }
            if (postalAddress.country) {
                [addressDict setObject:postalAddress.country forKey:@"country"];
            }
            if (postalAddress.street) {
                [addressDict setObject:postalAddress.street forKey:@"street"];
            }
            [addressDict setObject:[NSNumber numberWithInteger:0] forKey:@"indx"];
            [addressDict setObject:[NSNumber numberWithInteger:0] forKey:@"preference"];
            [addressArray addObject:addressDict];
            
        }
        
        [dictionary setObject:addressArray forKey:@"address"];
        
    }
    
    return [[PCContact alloc] initWithDictionary:dictionary];
    
}

-(NSString *)fetchServerSupportedString:(NSString *)string {
    
    if ([string rangeOfString:@"<"].location!=NSNotFound) {
        
        NSRange searchFromRange = [string rangeOfString:@"<"];
        NSRange searchToRange = [string rangeOfString:@">"];
        NSString *substring = [string substringWithRange:NSMakeRange(searchFromRange.location+searchFromRange.length, searchToRange.location-searchFromRange.location-searchFromRange.length)];
        return [substring lowercaseString];
        
    }else{
        
        return [string lowercaseString];
        
    }
}

- (NSString *)convertDateToServerSupportedFormat:(NSDate *)date {
    NSDateFormatter* serverFormat = [[NSDateFormatter alloc] init];
    [serverFormat setDateFormat:@"yyyy-MM-dd"];
    NSString *dateString = [serverFormat stringFromDate:date];
    return dateString;

}

@end
