//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import "ContactsDisplayViewController.h"
#import "ContactSearchViewController.h"
#import "UIViewController+Dismiss.h"
#import "UIViewController+RightBarButtonAction.h"
#import "UIViewController+Spinner.h"
#import "UIViewController+Alert.h"
#import "PCAccountInfo+additions.h"
#import "UIViewController+Contact.h"

@interface ContactSearchViewController (Private)

@property (nonatomic, strong) NSArray *contacts;

@end

@interface MediaGroupViewController ()
-(void) refreshContact;
@end

@interface ContactsDisplayViewController ()

@property (nonatomic, strong) NSArray *contacts;
@property (nonatomic, strong) UISearchController *searchController;
@property (nonatomic, strong) ContactSearchViewController *contactSearchViewController;
@property (strong, nonatomic) NSIndexPath *selectedIndexPath;

@end

@implementation ContactsDisplayViewController

- (void)viewDidLoad {
    [super viewDidLoad];

    UIStoryboard *storyBoard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    self.contactSearchViewController = (ContactSearchViewController *)[storyBoard instantiateViewControllerWithIdentifier: @"ContactSearchViewController"];
    self.contactSearchViewController.contacts = self.contacts;
    self.contactSearchViewController.searchMasterController = self;
    
    self.searchController = [[UISearchController alloc] initWithSearchResultsController:self.contactSearchViewController];
    self.searchController.searchResultsUpdater = self.contactSearchViewController;
    [self.searchController.searchBar sizeToFit];
    self.tableView.tableHeaderView = self.searchController.searchBar;
    self.searchController.dimsBackgroundDuringPresentation = NO;
    self.definesPresentationContext = YES;
    // Adding cancel button in view controller.
    [self addCancelButton];
    
    // Adding action items in RightBarButton.
    [self addRightBarButtonItemWithActions:[NSNumber numberWithInt:contactSearchRightBarAction],[NSNumber numberWithInt:firstnameRightBarAction],[NSNumber numberWithInt:lastnameRightBarAction],[NSNumber numberWithInt:versionRightBarAction],[NSNumber numberWithInt:idRightBarAction],[NSNumber numberWithInt:favoriteRightBarAction],[NSNumber numberWithInt:createdRightBarAction],[NSNumber numberWithInt:modifiedRightBarAction],[NSNumber numberWithInt:accountInfoRightBarAction], [NSNumber numberWithInt:signOutRightBarAction], nil];
    
    self.title = [NSString stringWithFormat:NSLocalizedString(@"%d Contacts", nil), self.contacts.count];
    [self.tableView scrollToRowAtIndexPath:self.selectedIndexPath atScrollPosition:UITableViewScrollPositionTop animated:NO];
}

- (NSArray *)dataSource {

    return self.contacts;
}

#pragma mark -
#pragma mark RightBarButtonAction callback

- (void)didBeginDoingRightBarButtonAction:(rightBarAction)action {
    
    [self showSpinner];
}

- (void)didFinishDoingRightBarButtonAction:(rightBarAction)action :(id)result :(id)response {
    
    [self hideSpinner];
    
    PCAPIQueryResponse *APIResponse = (PCAPIQueryResponse *)response;
    
    switch (APIResponse.urlResponse.statusCode) {
        case PCAPIResponseCodeOK: {
            
            switch (action) {
                case accountInfoRightBarAction: {
                    
                    PCAccountInfo *accountInfo = (PCAccountInfo *)result;
                    [self showOKAlertWithTitle:[self stringForRightBarAction:action]
                                    andMessage:[accountInfo humanReadableAccountInfo]
                              attributedString:nil
                                      okAction:nil];
                }
                    break;
                case firstnameRightBarAction:
                case lastnameRightBarAction:
                case createdRightBarAction:
                case modifiedRightBarAction:
                case versionRightBarAction:
                case favoriteRightBarAction:
                case contactSearchRightBarAction:
                case idRightBarAction:{
                    self.contacts = result;
                    self.contactSearchViewController.contacts = result;
                    self.title = [NSString stringWithFormat:NSLocalizedString(@"%d Contacts", nil), self.contacts.count];
                    [self.tableView reloadData];
                    if (self.contacts.count) {
                        [self.tableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0] atScrollPosition:0 animated:YES];
                    }
                    
                }
                    break;
                default:
                    break;
            }
            
        }
            break;
        case PCAPIResponseCodeUnauthorized:
            // API call failed because Session is no longer valid.
            [self showUnAuthorizedAlert];
            break;
        case PCAPIResponseCodeServiceUnavailable:
            // API call failed because server unavailable.
            [self showServerUnavailableAlert];
            break;
        default:
            // API call failed due to other network error.
            [self showOperationFailedAlert:[self stringForRightBarAction:action]];
            break;
    }
}

#pragma mark -
#pragma mark ContactAction delegates

- (void)didBeginDoingContactAction:(ContactAction)action {
    [self showSpinner];
}

- (void)didFinishDoingContactAction:(ContactAction)action onContact:(id)contact result:(id)result response:(id)response {
    
    PCAPIQueryResponse *APIResponse = (PCAPIQueryResponse *)response;
    
    [self hideSpinner];
    
    switch (APIResponse.urlResponse.statusCode) {
        case PCAPIResponseCodeResourceCreated:
        case PCAPIResponseCodeNoContent:
        case PCAPIResponseCodeOK: {
            
            switch (action) {
                case deleteContactAction: {
                    [self deleteContact:contact];
                }
                    break;
                default:
                    break;
            }
        }
            break;
        case PCAPIResponseCodeUnauthorized:
            [self showUnAuthorizedAlert];
            break;
        case PCAPIResponseCodeServiceUnavailable:
            [self showServerUnavailableAlert];
            break;
        default:
            [self showOperationFailedAlert:[self stringForContactAction:action]];
            break;
    }
}

#pragma mark -
#pragma mark ContactDetailViewController delegates

-(void) updateOldContact:(PCContact *)oldContact withUpdatedContact:(PCContact *)updatedContact {
    
    NSMutableArray *contacts = [NSMutableArray arrayWithArray:self.contacts];
    NSInteger index=[contacts indexOfObject:oldContact];
    [contacts replaceObjectsAtIndexes:[NSIndexSet indexSetWithIndex:index] withObjects:[NSArray arrayWithObject:updatedContact]];
    self.contacts = [NSArray arrayWithArray:contacts];
    self.title = [NSString stringWithFormat:NSLocalizedString(@"%d Contacts", nil), self.contacts.count];
    self.contactSearchViewController.contacts = self.contacts;
    [self.tableView reloadData];
    [_mediaViewController refreshContact];
}

-(void)deleteContact:(PCContact *)contact {
    
    NSMutableArray *contacts = [NSMutableArray arrayWithArray:self.contacts];
    [contacts removeObject:contact];
    self.contacts = [NSArray arrayWithArray:contacts];
    self.title = [NSString stringWithFormat:NSLocalizedString(@"%d Contacts", nil), self.contacts.count];
    [self.tableView reloadData];
    [_mediaViewController refreshContact];
    
}


@end
