//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import "MusicAlbumResourcesViewController.h"
#import "UIViewController+Alert.h"

@interface FileGridViewController (Protected)

@property (nonatomic, strong) UIRefreshControl *refreshControl;
@property (nonatomic, strong) UICollectionView *gridView;
@property (nonatomic, strong) NSString *emptyViewMessage;

- (void)refreshView;

@end

@implementation MusicAlbumResourcesViewController

- (void)viewDidLoad {
    
    [super viewDidLoad];
    
    self.title = self.album.name;
    self.emptyViewMessage = NSLocalizedString(@"No items for this Album", nil);
    [self.refreshControl beginRefreshing];
    self.gridView.alwaysBounceVertical = YES;
    [self reloadData];
}

- (void)reloadData {
    
    __weak MusicAlbumResourcesViewController *blocksafeSelf = self;
    [self.album getFileWithCompletionHandler:^(NSArray *albumFiles, PCAPIQueryResponse *response) {
        
        [blocksafeSelf.refreshControl endRefreshing];
        
        switch (response.urlResponse.statusCode) {
            case PCAPIResponseCodePreconditionFailed:
            case PCAPIResponseCodeOK: {
                
                blocksafeSelf.dataSource = [NSArray arrayWithArray:albumFiles];
                [blocksafeSelf refreshView];
            }
                break;
            case PCAPIResponseCodeUnauthorized:
                [blocksafeSelf showUnAuthorizedAlert];
                break;
            case PCAPIResponseCodeServiceUnavailable:
                [blocksafeSelf showServerUnavailableAlert];
                break;
            default:
                [blocksafeSelf showOperationFailedAlert:NSLocalizedString(@"Album related file", nil)];
                break;
        }
    }];
}

@end
