//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"

@interface PCAddPlaylistItemsAPITests : XCTestCase

@end

@implementation PCAddPlaylistItemsAPITests

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil.
 */
- (void)testNilAuthToken {
    
    // If User is not logged in, addPlaylistItems api should throw an exception
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCAddPlaylistItemsAPIParams *params = [[PCAddPlaylistItemsAPIParams alloc] init];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] addPlaylistItems:params withCompletionHandler:^(NSArray *playlistItems, PCAPIQueryResponse *response) {}],@"Get create addPlaylistItems API with nil access token");
    }
}

/*!
 * Test Case:PlaylistUid is required to addPlaylistItems response,otherwise test will fail.
 */

- (void)testForRequiredPlaylistUid {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"AddPlaylistItems API Result"];
        
        PCSearchAPIParams *musicSearchParams = [[PCSearchAPIParams alloc] init];
        musicSearchParams.query = [NSString setValue:@"audio*" forKey:@"contentType"];
        [musicSearchParams setSortKey:@"name" isAscending:YES];
        
        
        [[PCAPISDK sharedInstance] search:musicSearchParams withCompletionHandler:^(NSArray *results, PCAPIQueryResponse *response) {
            
            if (response.urlResponse.statusCode == PCAPIResponseCodeOK || response.urlResponse.statusCode == PCAPIResponseCodePreconditionFailed) {
                
                PCResource *resource = [results objectAtIndex:0];
                
                PCAddPlaylistItemsAPIParams *params = [[PCAddPlaylistItemsAPIParams alloc] init];
                params.paths = @[resource.fullPath];
                XCTAssertThrows([[PCAPISDK sharedInstance] addPlaylistItems:params withCompletionHandler:^(NSArray *playlistItems, PCAPIQueryResponse *response) {}],@"addPlaylistItems API throw exception without playlistUid");
                
            } else {
                
                XCTFail(@"Test Failed due to absence of music file");
            }
            [expectation fulfill];
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        
        XCTFail(@"Test Failed due to nil access token");
        
    }
    
}

/*!
 * Test Case:Paths is required to addPlaylistItems response,otherwise test will fail.
 */

- (void)testForRequiredPaths {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"AddPlaylistItems API Result"];
        
        PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];
        
        [[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {
            
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeOK: {
                    
                    XCTAssertGreaterThanOrEqual([playlist count], 0);
                    if (playlist.count) {
                        PCPlaylist *object = [playlist objectAtIndex:0];
                        PCAddPlaylistItemsAPIParams  *params = [[PCAddPlaylistItemsAPIParams alloc] init];
                        params.playlistUID = object.UID;
                        
                        XCTAssertThrows([[PCAPISDK sharedInstance] addPlaylistItems:params withCompletionHandler:^(NSArray *playlistItems, PCAPIQueryResponse *response) {}],@"addPlaylistItems API throw exception without paths");
                    }
                }
                    break;
                    
                default:
                   XCTFail(@"Test Failed due to absence of playlist");
                    break;
            }
            [expectation fulfill];
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        
        XCTFail(@"Test Failed due to nil access token");
        
    }
    
}

/*!
 * Test Case: If response code is 200 ,Playlist items array contains zero or greater than zero value. All playlist object has to be of class PCResource type and contains only object same as type of playlist. In all other cases (error), playlist has to be returned nil.
 */

- (void)testToAddItemsInsidePlaylist {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"AddPlaylistItems API Result"];
        
        PCSearchAPIParams *musicSearchParams = [[PCSearchAPIParams alloc] init];
        musicSearchParams.query = [NSString setValue:@"audio*" forKey:@"contentType"];
        [musicSearchParams setSortKey:@"name" isAscending:YES];
        
        [[PCAPISDK sharedInstance] search:musicSearchParams withCompletionHandler:^(NSArray *results, PCAPIQueryResponse *response) {
            
            if (response.urlResponse.statusCode == PCAPIResponseCodeOK || response.urlResponse.statusCode == PCAPIResponseCodePreconditionFailed) {
                
                if (results.count) {
                    PCResource *resource = [results objectAtIndex:0];
                    NSMutableArray *filePathArray = [NSMutableArray new];
                    [filePathArray addObject:@[resource.fullPath]];
                    
                    PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];
                    params.type = @"music";
                    [[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {
                        switch (response.urlResponse.statusCode) {
                            case PCAPIResponseCodeOK: {
                                if (playlist.count) {
                                    PCPlaylist *object = [playlist objectAtIndex:0];
                                    
                                    PCAddPlaylistItemsAPIParams  *params = [[PCAddPlaylistItemsAPIParams alloc] init];
                                    params.playlistUID = object.UID;
                                    params.paths = filePathArray;
                                    
                                    [[PCAPISDK sharedInstance] addPlaylistItems:params withCompletionHandler:^(NSArray *playlistItems, PCAPIQueryResponse *response) {
                                        switch (response.urlResponse.statusCode) {
                                            case PCAPIResponseCodeOK: {
                                                
                                                XCTAssertGreaterThanOrEqual([playlistItems count], 0);
                                            }
                                                break;
                                                
                                            default:
                                                XCTAssertNil(playlistItems,@"Playlist items array return nil");
                                                break;
                                        }
                                        [expectation fulfill];
                                    }];
                                    
                                } else {
                                    XCTFail(@"Test Failed due to absence of playlistUid");
                                }
                            }
                                break;
                                
                            default:
                                XCTFail(@"Test Failed due to absence of playlist");
                                break;
                        }
                        [expectation fulfill];
                    }];
                    
                } else {
                    XCTFail(@"Test Failed : file is not available");
                }
            } else {
                
                XCTFail(@"Test Failed due to absence of music file");
            }
            [expectation fulfill];
        }];
        
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*3 handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        
        XCTFail(@"Test Failed due to nil access token");
        
    }
}

@end
