//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"

@interface PCDeleteContactAPITests : XCTestCase

@end

@implementation PCDeleteContactAPITests

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil.
 */
- (void)testNilAuthToken {
    
    // If User is not logged in, Delete Contact api should throw an exception
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCDeleteContactAPIParams *params = [[PCDeleteContactAPIParams alloc] init];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] deleteContact:params withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Delete Contact API with nil access token");
    }
    
}

/*!
 * Test Case:contactId is required to delete Contact,otherwise test will fail.
 */

- (void)testForRequiredContactID {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCDeleteContactAPIParams* params = [[PCDeleteContactAPIParams alloc] init];
        XCTAssertThrows([[PCAPISDK sharedInstance] deleteContact:params withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Delete Contact API required contact ID");
        
    } else {
        XCTFail(@"Test failed due to nil access token");
    }
}

/*!
 * Test Case: If response code is 204, delete contact API will delete provided contact. In all other cases (error),it will not delete contact.
 */

- (void)testToDeleteContact {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Fetch Contact result"];
        
        PCContactAPIParams *params = [[PCContactAPIParams alloc] init];
        
        [[PCAPISDK sharedInstance] fetchContacts:params withCompletionHandler:^(NSArray *contacts, PCAPIQueryResponse *response) {
            switch (response.urlResponse.statusCode) {
                    
                case PCAPIResponseCodeOK: {
                    
                    XCTAssertGreaterThanOrEqual([contacts count], 0);
                    
                    if (contacts.count) {
                        
                        PCContact *object = [contacts objectAtIndex:0];
                        
                        PCDeleteContactAPIParams *params = [[PCDeleteContactAPIParams alloc] init];
                        params.contactID = object.ID;
                        
                        [[PCAPISDK sharedInstance] deleteContact:params withCompletionHandler:^(PCAPIQueryResponse *response) {
                            switch (response.urlResponse.statusCode) {
                                case PCAPIResponseCodeNoContent:
                                    XCTAssertTrue([object isMemberOfClass:[PCContact class]],@"Delete contact API return PCContact object");
                                    break;
                                default:
                                    XCTFail(@"Delete contact API failed");
                                    break;
                            }
                            [expectation fulfill];
                        }];
                    }
                    
                }
                    break;
                    
                default:
                    XCTFail(@"Test Failed due to absence of contacts");
                    break;
            }
            [expectation fulfill];
        }];
        
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
            
        }];
    } else {
        XCTFail(@"Test failed due to nil access token");
    }
}

@end
