//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"

@interface PCDeletePlaylistItemsAPITests : XCTestCase

@end

@implementation PCDeletePlaylistItemsAPITests

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil.
 */
- (void)testNilAuthToken {
    
    // If User is not logged in, Delete Playlist api should throw an exception
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCDeletePlaylistItemAPIParams *params = [[PCDeletePlaylistItemAPIParams alloc] init];
        XCTAssertThrows([[PCAPISDK sharedInstance] deletePlaylistItem:params withCompletionHandler:^(PCAPIQueryResponse *response){}],@"Delete playlist items API with nil access token");
        
    }
    
}

/*!
 * Test Case:PlaylistID is required to delete playlist items response,otherwise test will fail.
 */

- (void)testForRequiredPlaylistID {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Delete playlist items API result"];
        
        PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];
        
        [[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {
            
            switch (response.urlResponse.statusCode) {
                    
                case PCAPIResponseCodeOK: {
                    
                    XCTAssertGreaterThanOrEqual([playlist count], 0);
                    
                    if (playlist.count) {
                        
                        PCPlaylist *object = [playlist objectAtIndex:0];
                        
                        PCDeletePlaylistItemAPIParams* params = [[PCDeletePlaylistItemAPIParams alloc] init];
                        
                        if ([object.paths count]) {
                            params.itemUID = [[object.paths objectAtIndex:0] objectForKey:@"itemuid"];
                        } else {
                            XCTFail(@"Test Failed due to absence of playlist item ID");
                        }
                        
                        XCTAssertThrows([[PCAPISDK sharedInstance] deletePlaylistItem:params withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Delete playlist items API required playlist ID");
                    }
                    
                }
                    break;
                    
                default:
                    XCTFail(@"Test Failed due to absence of playlist");
                    break;
            }
            [expectation fulfill];
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
            
        }];
        
    } else {
        
        XCTFail(@"Test failed due to nil access token");
        
    }
}

/*!
 * Test Case:ItemUid is required to delete playlist items response,otherwise test will fail.
 */

- (void)testForRequiredItemUid {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Delete playlist items API result"];
        
        PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];
        
        [[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {
            
            switch (response.urlResponse.statusCode) {
                    
                case PCAPIResponseCodeOK: {
                    
                    XCTAssertGreaterThanOrEqual([playlist count], 0);
                    
                    if (playlist.count) {
                        
                        PCPlaylist *object = [playlist objectAtIndex:0];
                        
                        PCDeletePlaylistItemAPIParams* params = [[PCDeletePlaylistItemAPIParams alloc] init];
                        params.playlistUID = object.UID;
                        
                        XCTAssertThrows([[PCAPISDK sharedInstance] deletePlaylistItem:params withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Delete playlist items API required Item UID");
                    }
                    
                }
                    break;
                    
                default:
                    XCTFail(@"Test Failed due to absence of playlist");
                    break;
            }
            [expectation fulfill];
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
            
        }];
        
    } else {
        
        XCTFail(@"Test failed due to nil access token");
        
    }
}

/*!
 * Test Case: If response code is 200, delete playlist item API delete provided playlist item. In all other cases (error),it will not delete playlist items.
 */

- (void)testToDeletePlaylistItems {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Delete playlist items API result"];
        
        PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];
        
        [[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {
            
            switch (response.urlResponse.statusCode) {
                    
                case PCAPIResponseCodeOK: {
                    
                    XCTAssertGreaterThanOrEqual([playlist count], 0);
                    
                    if (playlist.count) {
                        
                        PCPlaylist *object = [playlist objectAtIndex:0];
                        
                        PCDeletePlaylistItemAPIParams* params = [[PCDeletePlaylistItemAPIParams alloc] init];
                        params.playlistUID = object.UID;
                        
                        if ([object.paths count]) {
                            params.itemUID = [[object.paths objectAtIndex:0] objectForKey:@"itemuid"];
                        } else {
                            XCTFail(@"Test Failed due to absence of playlist item ID");
                        }
                        
                        [[PCAPISDK sharedInstance] deletePlaylistItem:params withCompletionHandler:^(PCAPIQueryResponse *response) {
                            
                            switch (response.urlResponse.statusCode) {
                                case PCAPIResponseCodeOK:
                                    XCTAssertTrue([playlist isMemberOfClass:[PCPlaylist class]],@"Delete playlist item API return PCPlaylist object");
                                    break;
                                default:
                                    XCTFail(@"Delete playlist item API failed");
                                    break;
                            }
                            [expectation fulfill];
                        }];
                    }
                    
                }
                    break;
                    
                default:
                    XCTFail(@"Test Failed due to absence of playlist");
                    break;
            }
            [expectation fulfill];
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
            
        }];
        
    } else {
        
        XCTFail(@"Test failed due to nil access token");
        
    }

}

@end
