//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"


@interface PCFilesAPITests : XCTestCase

@end

@implementation PCFilesAPITests

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil
 */
- (void)testNilAuthToken {
    // If User is not logged in, files api should throw an exception
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCFilesAPIParams *params = [[PCFilesAPIParams alloc] init];
        XCTAssertThrows([[PCAPISDK sharedInstance] downloadFile:params progress:^(long long totalBytesDownloaded, long long totalBytesToBeDownloaded) {} withCompletionHandler:^(PCAPIQueryResponse *response) {}] , @"Files API with nil access token");
        
    }
}

/*!
 * Test Case: path is a required parameter. SDK should throw an exception in case path is nil.
 */
- (void)testNilPath {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Download File API"];
        
        PCSearchAPIParams *photoSearchParams = [[PCSearchAPIParams alloc] init];
        photoSearchParams.query = [NSString setValue:@"image*" forKey:@"contentType"];
        [photoSearchParams setSortKey:@"name" isAscending:YES];
        
        [[PCAPISDK sharedInstance] search:photoSearchParams withCompletionHandler:^(NSArray *results, PCAPIQueryResponse *response) {
            if (results.count) {
                
                NSManagedObjectID *objID = [results objectAtIndex:0];
                PCResource *resource = [[PCResource alloc] initWithDBResourceID:objID];
                
                PCFilesAPIParams *params = [[PCFilesAPIParams alloc] init];
                params.path = nil;
                
                XCTAssertThrows([[PCAPISDK sharedInstance] downloadFile:params progress:^(long long totalBytesDownloaded, long long totalBytesToBeDownloaded) {} withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"File API with nil path");
            } else {
                XCTFail(@"Test could not be completed");
            }
            [expectation fulfill];
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) { // time*2 because 2 api calls exectued in test
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}

/*!
 * Test Case: size is a required parameter. SDK should throw an exception in case size is nil.
 */
- (void)testNilSize {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Download File API"];
        
        PCSearchAPIParams *photoSearchParams = [[PCSearchAPIParams alloc] init];
        photoSearchParams.query = [NSString setValue:@"image*" forKey:@"contentType"];
        [photoSearchParams setSortKey:@"name" isAscending:YES];
        
        [[PCAPISDK sharedInstance] search:photoSearchParams withCompletionHandler:^(NSArray *results, PCAPIQueryResponse *response) {
            if (results.count) {
                
                NSManagedObjectID *objID = [results objectAtIndex:0];
                PCResource *resource = [[PCResource alloc] initWithDBResourceID:objID];
                
                PCFilesAPIParams *params = [[PCFilesAPIParams alloc] init];
                params.path = resource.fullPath;
                
                XCTAssertThrows([[PCAPISDK sharedInstance] downloadFile:params progress:^(long long totalBytesDownloaded, long long totalBytesToBeDownloaded) {} withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"File API with nil size");
            } else {
                XCTFail(@"Test could not be completed");
            }
            [expectation fulfill];
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) { // time*2 because 2 api calls exectued in test
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}

/*!
 * Test Case:SDK should return a image when user will pass valid path.
 */
- (void)testFilesAPIResponse {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Download File API"];
        
        PCSearchAPIParams *photoSearchParams = [[PCSearchAPIParams alloc] init];
        photoSearchParams.query = [NSString setValue:@"image*" forKey:@"contentType"];
        [photoSearchParams setSortKey:@"name" isAscending:YES];
        
        [[PCAPISDK sharedInstance] search:photoSearchParams withCompletionHandler:^(NSArray *results, PCAPIQueryResponse *response) {
            if (results.count) {
                
                NSManagedObjectID *objID = [results objectAtIndex:0];
                PCResource *resource = [[PCResource alloc] initWithDBResourceID:objID];
                
                PCFilesAPIParams *params = [[PCFilesAPIParams alloc] init];
                params.path = resource.fullPath;
                
                [[PCAPISDK sharedInstance] downloadFile:params progress:^(long long totalBytesDownloaded, long long totalBytesToBeDownloaded) {} withCompletionHandler:^(PCAPIQueryResponse *response) {
                    NSData *fileData    = [NSData dataWithContentsOfURL:response.downloadedURL];
                    UIImage *image = [UIImage imageWithData:fileData];
                    
                    switch (response.urlResponse.statusCode) {
                        case PCAPIResponseCodeOK:
                        case PCAPIResponseCodePreconditionFailed: {
                            // check if image able to download it will return image.
                            XCTAssertNotNil(image, @"downloaded file not nil");
                        }
                            break;
                            
                        default:{
                            // check if image not able to download it will return nil image.
                            XCTAssertNil(image, @"Fail to download return nil image");
                        }
                            break;
                    }
                    
                    [expectation fulfill];
                }];
            } else {
                XCTFail(@"Test could not be completed");
            }
            [expectation fulfill];
        }];
        
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) { // time*2 because 2 api calls exectued in test
            
            if (error) {
                NSLog(@"error: %@", error);
            }
        }];
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}




@end
