//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"

@interface PCFullViewAPITests : XCTestCase

@end

@implementation PCFullViewAPITests

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    [super tearDown];
    // Put teardown code here. This method is called after the invocation of each test method in the class.
}

/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil.
 */
- (void)testNilAuthToken {
    // If User is not logged in, fullSync should throw an exception
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        XCTAssertThrows([[PCAPISDK sharedInstance] doFullSyncWithCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Full sync with nil access token");
    }
}

/*!
 * Test Case: If response code is 200, full sync has to non nil value. Also full sync objects has to be of class PCResource type. In all other cases (error), full sync has to be returned nil
 */
- (void)testFullViewAPIResponse {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {

        XCTestExpectation *expectation = [self expectationWithDescription:@"Full sync API Result"];
        
        [[PCAPISDK sharedInstance] doFullSyncWithCompletionHandler:^(PCAPIQueryResponse *response) {
            
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodePreconditionFailed:
                case PCAPIResponseCodeOK: {
                    PCSearchAPIParams *photoSearchParams = [[PCSearchAPIParams alloc] init];
                    photoSearchParams.query = [NSString setValue:@"image*" forKey:@"contentType"];
                    [photoSearchParams setSortKey:@"name" isAscending:YES];
                    
                    [[PCAPISDK sharedInstance] search:photoSearchParams withCompletionHandler:^(NSArray *results, PCAPIQueryResponse *response) {
                        
                        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^(void) {
                            
                            NSMutableArray *searchResultObjects = [NSMutableArray new];
                            
                            for (NSManagedObjectID *objID in results) {
                                PCResource *resource = [[PCResource alloc] initWithDBResourceID:objID];
                                [searchResultObjects addObject:resource];
                            }
                            
                             XCTAssertNotNil(searchResultObjects, @"Full sync not return nil value");
                        });
                        [expectation fulfill];
                        
                    }];
                    
                    PCSearchAPIParams *videoSearchParams = [[PCSearchAPIParams alloc] init];
                    videoSearchParams.query = [NSString setValue:@"video*" forKey:@"contentType"];
                    [videoSearchParams setSortKey:@"name" isAscending:YES];
                    
                    [[PCAPISDK sharedInstance] search:videoSearchParams withCompletionHandler:^(NSArray *results, PCAPIQueryResponse *response) {
                        
                        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^(void) {
                            
                            NSMutableArray *searchResultObjects = [NSMutableArray new];
                            
                            for (NSManagedObjectID *objID in results) {
                                PCResource *resource = [[PCResource alloc] initWithDBResourceID:objID];
                                [searchResultObjects addObject:resource];
                            }
                            
                            XCTAssertNotNil(searchResultObjects, @"Full sync not return nil value");
                        });
                        [expectation fulfill];
                    }];
                    
                    PCSearchAPIParams *musicSearchParams = [[PCSearchAPIParams alloc] init];
                    musicSearchParams.query = [NSString setValue:@"audio*" forKey:@"contentType"];
                    [musicSearchParams setSortKey:@"name" isAscending:YES];
                    
                    [[PCAPISDK sharedInstance] search:musicSearchParams withCompletionHandler:^(NSArray *results, PCAPIQueryResponse *response) {
                        
                        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^(void) {
                            
                            NSMutableArray *searchResultObjects = [NSMutableArray new];
                            
                            for (NSManagedObjectID *objID in results) {
                                PCResource *resource = [[PCResource alloc] initWithDBResourceID:objID];
                                [searchResultObjects addObject:resource];
                            }
                            
                            XCTAssertNotNil(searchResultObjects, @"Full sync not return nil value");
                        });
                        [expectation fulfill];
                    }];
                }
                    break;
                default:
                    XCTFail(@"Test failed due to full view API failed");
                    break;
            }
            [expectation fulfill];

        }];
        
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*4 handler:^(NSError *error) {
           
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
    } else {
        
          XCTFail(@"Test failed due to nil access token");
    }
}

@end
