//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"

@interface PCGetPlaylistDefinitionAPITests : XCTestCase

@end

@implementation PCGetPlaylistDefinitionAPITests

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil.
 */
- (void)testNilAuthToken {
    // If User is not logged in, get playlist definition api should throw an exception
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCGetPlaylistDefinitionAPIParams *params = [[PCGetPlaylistDefinitionAPIParams alloc] init];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] getPlaylistDefinition:params withCompletionHandler:^(PCPlaylist *playlist, PCAPIQueryResponse *response) {}],@"Get playlist definition API with nil access token");
    }
}

/*!
 * Test Case: Playlist ID is required parameter. SDK should throw and exception in case playlist ID is nil.
 */
- (void)testForRequiredPlaylistUid {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCGetPlaylistDefinitionAPIParams *params = [[PCGetPlaylistDefinitionAPIParams alloc] init];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] getPlaylistDefinition:params withCompletionHandler:^(PCPlaylist *playlist, PCAPIQueryResponse *response) {}],@"Get playlist definition API without playlist ID");
        
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}

/*!
 * Test Case: If response code is 200. Playlist items object has to be of class PCPlaylist type. In all other cases (error), playlist object has to be returned nil.
 */

- (void)testGetPlaylistDefinition {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
         XCTestExpectation *expectation = [self expectationWithDescription:@"Get playlist definition API Result"];
        
        PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];

        [[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {
            
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeOK: {
                    XCTAssertGreaterThanOrEqual([playlist count], 0);
                    
                    if (playlist.count) {
                        PCPlaylist *object = [playlist objectAtIndex:0];
                        
                        PCGetPlaylistDefinitionAPIParams *params = [[PCGetPlaylistDefinitionAPIParams alloc] init];
                        params.playlistUID = object.UID;
                        
                        [[PCAPISDK sharedInstance] getPlaylistDefinition:params withCompletionHandler:^(PCPlaylist *playlist, PCAPIQueryResponse *response) {
                            switch (response.urlResponse.statusCode) {
                                case PCAPIResponseCodeOK:
                                    XCTAssertTrue([playlist isMemberOfClass:[PCPlaylist class]],@"Get playlist definition API return PCPlaylist object");
                                    break;
                                    
                                default:
                                     XCTFail(@"Get playlist definition API failed");
                                    break;
                            }
                            [expectation fulfill];
                            
                        }];
                    }
                }
                    break;
                    
                default:
                    XCTAssertNil(playlist,@"Playlist array return nil");
                    break;
            }
            [expectation fulfill];
            
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        
        XCTFail(@"Test Failed due to nil access token");
        
    }

}


@end
