//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import <Foundation/Foundation.h>
#import "PCAPILoggerProtocol.h"

/*!
 *PCAPILogger class used to start, stop, add and delete diffrent log levels. Third party applications can invoke all method's using
 public methods exposed in this class.
 */

@interface PCAPILogger : NSObject

@property (nonatomic, strong, readonly) NSSet <PCAPILoggerProtocol> *loggers;

/*!
 * Returns the shared logger instance.
 */
+ (instancetype)sharedLogger;

/*!
 * Start logging to all managed loggers.
 */
- (void)startLogging;

/*!
 * Stop logging to all managed loggers.
 */
- (void)stopLogging;

/*!
 * Adds the given logger to be managed to the `loggers` set.
 */
- (void)addLogger:(id <PCAPILoggerProtocol>)logger;

/*!
 * Removes the given logger from the `loggers` set.
 */
- (void)removeLogger:(id <PCAPILoggerProtocol>)logger;

/*!
 * Removes all logger from the `loggers` set.
 */
- (void)removeAllLogger;


@end
