//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import "PCResourceAPIParams.h"

typedef enum {
    firstnameContactSortType,
    lastnameContactSortType,
    createdContactSortType,
    modifiedContactSortType,
    favoriteContactSortType,
    versionContactSortType,
    idContactSortType
} ContactSortType;

/*!
 * PCContactAPIParams stores all the parameters required to fetch contacts from user account.
 */
@interface PCContactAPIParams : PCResourceAPIParams

/*!
 * Page number to return, for paginated responses. Defaulted to 1.
 */
@property (nonatomic, assign) int page;

/*!
 * Maximum children to include in a paginated response. Defaulted to 200.
 */
@property (nonatomic, assign) int count;

/*!
 * Specify sort order for response. Syntax is :'{field}+{asc|desc}'. Valid values for 'field' are: firstname, lastname, created, modified, id, version and favorite.
 */
@property (nonatomic, copy, readonly) NSString *sort;

/*!
 * Contains the text to be searched to particular parts of a contact. Supports following fields: id, name, email, im, address, tel, field-tag.type (Example: address.home), address.type.subfield (Example: address.home.street), favorite (Example: favorite:true and favorite:false)
 */
@property (nonatomic, readonly) NSString *query;

/*!
 * Call this method to set sort order for contact results.
 * \param sortType the field on which sort has to be done.
 * \param ascending if true, results come in ascending order else result comes in descending order.
 */
- (void)setSortKey:(ContactSortType)sortType isAscending:(BOOL)ascending;

/*!
 * Call this method to set query for contact results.
 * \param value to be searched to particular parts of a contact.
 * \param  field  on which search has to be done. Supports following fields: id, name, email, im, address, tel, field-tag.type (Example: address.home), address.type.subfield (Example: address.home.street), favorite (Example: favorite:true and favorite:false)
 */
- (void)setQueryValue:(NSString *)value forField:(NSString *)field;

@end
