//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import "WebViewController.h"
#import "PCResource+Info.h"
#import "UIViewController+Spinner.h"
#import "UIViewController+Alert.h"

@interface WebViewController ()
@property (weak, nonatomic) IBOutlet UIWebView *webView;

@end

@implementation WebViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.title = self.selectedResource.name;
    
    [self showSpinner];
    
    // Intialize the PCFilesAPIParams object.
    PCFilesAPIParams *params = [[PCFilesAPIParams alloc] init];
    params.path = self.selectedResource.fullPath;
    
    __weak WebViewController *blocksafeSelf = self;
    
    /*
     @Description :
     This API is used to download a file. Once the download process is succeed/failed result will come with completion block.
     
     @Parameters :
     PCFilesAPIParams:
     * This class stores all the parameters required to download a file. It contains attributes such as path.
     * For providing access token to SDK user have to logged In.
     * path is required parameter to call this API.
     * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
     * More information about parameters you will find in PCFilesAPIParams.h class.
     
     @CompletionHandler :
     This completion block gets called once the download process is succeed/failed. completion block contains below parameter.
     PCAPIQueryResponse : Stores all the server response and error for this API. dowloaded result comes in downloadedURL attribute.The file is downloaded to a temporary path and the file will be removed once the api which downloaded the file is released. So move the file to a different location in completion callback.
     
     @To download thumbnail image of resource, you must take the following steps:
     * Include VerizonCloudAPI.framework header file in your class.
     #import <VerizonCloudAPI/VerizonCloudAPI.h>
     * Initalize the PCFilesAPIParams class and provide the required parameter.
     * Initalize the PCAPISDK object and call the  downloadFile:progress:withCompletionHandler: API.
     * Perform the  downloadFile:progress:withCompletionHandler: API call using the code sample below.
     * Get the result (succeed or failed) in completion block.
     * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
     */
    
    // Initilize the PCAPISDK and call the API.
    
    [[PCAPISDK sharedInstance] downloadFile:params progress:^(long long totalBytesDownloaded, long long totalBytesToBeDownloaded) {
        // Get the progress in this block.
        
        
    } withCompletionHandler:^(PCAPIQueryResponse *response) {
       // Get the result in this block.
        [blocksafeSelf hideSpinner];
        switch (response.urlResponse.statusCode) {
            case PCAPIResponseCodeOK: {
                
                NSData *fileData = [NSData dataWithContentsOfURL:response.downloadedURL];
                if (fileData) {
                    [blocksafeSelf.webView loadData:fileData MIMEType:self.selectedResource.mimeType textEncodingName:@"UTF-8" baseURL:nil];
                }
            }
                break;
            case PCAPIResponseCodeUnauthorized:
                [blocksafeSelf showUnAuthorizedAlert];
                break;
            case PCAPIResponseCodeServiceUnavailable:
                [blocksafeSelf showServerUnavailableAlert];
                break;
            default:
                [blocksafeSelf showOperationFailedAlert:NSLocalizedString(@"Get File", nil)];
                break;
        }
    }];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
