//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import "ZoomableImageGridCell.h"

#define MAXIMUM_SCALE           6.0
#define INTERMEDIATE_SCALE      3.0
#define MINIMUM_SCALE           1.0

@interface ZoomableImageGridCell ()

@property (weak, nonatomic) id <ZoomableImageGridCellDelegate> gridCellDelegate;

@end

@implementation ZoomableImageGridCell

@dynamic gridCellDelegate;

- (void)awakeFromNib {
    
    [super awakeFromNib];
    
    UITapGestureRecognizer *recognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(singleTapped:)];
    [recognizer setNumberOfTapsRequired:1];
    [self.scrollView addGestureRecognizer:recognizer];
    
    self.scrollView.maximumZoomScale = MAXIMUM_SCALE;
    self.scrollView.minimumZoomScale = MINIMUM_SCALE;
    
    UITapGestureRecognizer *doubleTap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(doubleTapped:)];
    [doubleTap setNumberOfTapsRequired:2];
    [self.scrollView addGestureRecognizer:doubleTap];
}

- (void)singleTapped:(UILongPressGestureRecognizer *)gestureRecognizer {
    
    if (gestureRecognizer.state == UIGestureRecognizerStateEnded) {
        if ([self.gridCellDelegate respondsToSelector:@selector(gridCellDidSelected:)]) {
            [self.gridCellDelegate gridCellDidSelected:self];
        }
    }
}

- (void)doubleTapped:(UILongPressGestureRecognizer *)gestureRecognizer {
    
    if(self.scrollView.zoomScale >= INTERMEDIATE_SCALE) {
        
        [self.scrollView setZoomScale:MINIMUM_SCALE animated:YES];
    } else {
        
        [self.scrollView setZoomScale:INTERMEDIATE_SCALE animated:YES];
    }
}

- (UIView *)viewForZoomingInScrollView:(UIScrollView *)scrollView {
    return self.imageView;
}

@end
