//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import "PCFile+Info.h"
#import <UIKit/UIKit.h>

static NSDateFormatter *dateFormatterForDisplay;

@implementation PCFile (Info)

// Create a path to cache thumbnail image for this file.
- (NSString *)thumbnailCachePath {
    
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString *thumbnailImagePath = [documentsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"Thumbs_%@", self.contentToken]];
    return thumbnailImagePath;
}

// Create a path to cache download image for this file.
- (NSString *)fileCachePath {
    
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString *filePath = [documentsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"File_%@", self.contentToken]];
    return filePath;
}

// Using mimeType return type of the file. like image, audio, video etc.
- (fileType)fileType {
    
    NSString *mimeType = self.mimeType;
    
    if ([mimeType containsString:@"image"]) {
        return imageFileType;
    } else if ([mimeType containsString:@"audio"]) {
        return musicFileType;
    } else if ([mimeType containsString:@"video"]) {
        return videoFileType;
    } else if ([mimeType containsString:@"application"]){
        return documentFileType;
    }
    return unknowFileType;
}

 // Convert PCFile to NSAttributedString to display in UI.
- (NSAttributedString *)fileInfoWithTags:(PCTags*)tag{
    
    if (!dateFormatterForDisplay) {
        dateFormatterForDisplay = [PCFile dateFormatterForDisplay];
    }
    
    NSString *mimeType = self.mimeType;
    
    
    NSMutableString *fileInfoString = [NSMutableString stringWithFormat:@"%C NAME: %@\n%C PARENTPATH: %@\n%C VERSION CREATED: %@\n%C EXTENSION: %@\n%C MIME TYPE: %@\n%C SIZE: %@\n", (unichar) 0x2022, self.name,(unichar) 0x2022, self.parentPath,(unichar) 0x2022, [dateFormatterForDisplay stringFromDate:self.versionCreated], (unichar) 0x2022, [self.name pathExtension],(unichar) 0x2022, mimeType, (unichar) 0x2022, [self transformedValue:[self.size longLongValue]]];
    
    if (self.fileType == imageFileType) {
        
        PCPhoto *photo = (PCPhoto *)self;
        
        [fileInfoString appendFormat:@"%C CAPTURED DATE: %@\n", (unichar) 0x2022, photo.capturedDate?[dateFormatterForDisplay stringFromDate:photo.capturedDate]:NSLocalizedString(@"Unknown Date", nil)];
        
    } else if (self.fileType == musicFileType) {
        
        PCAudio *audio = (PCAudio *)self;
        NSString *artist = audio.artist;
        NSString *album = audio.album;
        
        [fileInfoString appendFormat:@"%C ARTIST: %@\n", (unichar) 0x2022, artist?artist:NSLocalizedString(@"Unknown Artist", nil)];
        [fileInfoString appendFormat:@"%C ALBUM: %@\n", (unichar) 0x2022, album?album:NSLocalizedString(@"Unknown Album", nil)];
        
    } else if (self.fileType == videoFileType) {
        
        PCVideo *video = (PCVideo *) self;
        NSString *artist = video.artist;
        NSString *album = video.album;
        
        [fileInfoString appendFormat:@"%C CAPTURED DATE: %@\n", (unichar) 0x2022, video.capturedDate?[dateFormatterForDisplay stringFromDate:video.capturedDate]:NSLocalizedString(@"Unknown Date", nil)];
        [fileInfoString appendFormat:@"%C ARTIST: %@\n", (unichar) 0x2022, artist?artist:NSLocalizedString(@"Unknown Artist", nil)];
        [fileInfoString appendFormat:@"%C ALBUM: %@\n", (unichar) 0x2022, album?album:NSLocalizedString(@"Unknown Album", nil)];
    }
    if (tag.tags){
        [fileInfoString appendFormat:@"%C Tags: %@\n", (unichar) 0x2022,tag.tags];
    }
    
    NSMutableParagraphStyle *paragraphStyle = [NSMutableParagraphStyle new];
    paragraphStyle.alignment = NSTextAlignmentLeft;
    
    NSRange wholeRange = NSMakeRange(0, fileInfoString.length);
    
    NSMutableAttributedString *attributedString = [[NSMutableAttributedString alloc] initWithString:fileInfoString];
    [attributedString addAttribute:NSParagraphStyleAttributeName value:paragraphStyle range:wholeRange];
    [attributedString addAttribute:NSFontAttributeName value:[UIFont systemFontOfSize:12.0] range:wholeRange];
    
    return attributedString;
}

#pragma mark
#pragma mark -
// Creating NSDateFormatter for "dd-MMM-yyyy'".
+ (NSDateFormatter *)dateFormatterForDisplay {
    NSDateFormatter *dateFormat = [[NSDateFormatter alloc] init];
    [dateFormat setDateFormat:@"dd-MMM-yyyy'"];
    return dateFormat;
}

// Using size attribute converting value in any of these "bytes", "KB", "MB", "GB" and "TB".
- (NSString *)transformedValue:(long long)value {
    
    double convertedValue = (double)value;
    int multiplyFactor = 0;
    
    NSArray *tokens = [NSArray arrayWithObjects:@"bytes",@"KB",@"MB",@"GB",@"TB",nil];
    
    while (convertedValue > 1024) {
        convertedValue /= 1024;
        multiplyFactor++;
    }
    
    return [NSString stringWithFormat:@"%4.2f %@",convertedValue, [tokens objectAtIndex:multiplyFactor]];
}


@end
