//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import "UIViewController+RightBarButtonAction.h"
#import "SearchConfigurationViewController.h"
#import "UIViewController+Alert.h"
#import "PCAccountInfo+additions.h"
#import "UIViewController+Dismiss.h"
#import "UIViewController+Alert.h"
#import <MessageUI/MessageUI.h>
#ifdef KEPLER_FILE_BROWSER
#import <MobileCoreServices/MobileCoreServices.h>
#import <AssetsLibrary/AssetsLibrary.h>
#import "UploadManager.h"
#import "FavoritesViewController.h"
#endif
#ifdef HUBBLE_MEDIA_BROWSER
#import "SharesTableViewController.h"
#import "CreateContactViewController.h"
#endif

@interface UIViewController () <UIImagePickerControllerDelegate, UINavigationControllerDelegate , MFMailComposeViewControllerDelegate>

@end

@interface ActionBarButtonItem : UIBarButtonItem

@property (nonatomic, strong) NSArray *actions;

@end

@implementation ActionBarButtonItem


@end


@implementation UIViewController (RightBarButtonAction)

- (NSString *)stringForRightBarAction:(rightBarAction)action {

    switch (action) {
        case accountInfoRightBarAction:
            return NSLocalizedString(@"Account Info", nil);
            break;
        case searchRightBarAction:
            return NSLocalizedString(@"Search", nil);
            break;
#ifdef KEPLER_FILE_BROWSER
        case emptyTrashRightBarAction:
            return NSLocalizedString(@"Empty Trash", nil);
            break;
        case createFolderRightBarAction:
            return NSLocalizedString(@"Create Folder", nil);
            break;
        case createVirtualFolderRightBarAction:
            return NSLocalizedString(@"Create Virtual Folder", nil);
            break;
        case getFavoritesRightBarAction:
            return NSLocalizedString(@"Get Favorites", nil);
            break;
        case uploadFileRightBarAction:
            return NSLocalizedString(@"Upload File", nil);
            break;
        case photoMediaRightBarAction:
            return NSLocalizedString(@"Photos", nil);
            break;
        case videoMediaRightBarAction:
            return NSLocalizedString(@"Videos", nil);
            break;
        case audioMediaRightBarAction:
            return NSLocalizedString(@"Audio", nil);
            break;
        case docMediaRightBarAction:
            return NSLocalizedString(@"Document", nil);
            break;
        case videoFavoriteRightBarAction:
            return NSLocalizedString(@"Video", nil);
            break;
        case audioFavoriteRightBarAction:
            return NSLocalizedString(@"Music", nil);
            break;
        case photoFavoriteRightBarAction:
            return NSLocalizedString(@"Image", nil);
            break;
        case docFavoriteRightBarAction:
            return NSLocalizedString(@"Documents", nil);
            break;
        case folderFavoriteRightBarAction:
            return NSLocalizedString(@"Folder", nil);
            break;
        case fileFavoriteRightBarAction:
            return NSLocalizedString(@"File", nil);
            break;
        case allFavoriteRightBarAction:
            return NSLocalizedString(@"All", nil);
            break;
            
#endif
#ifdef HUBBLE_MEDIA_BROWSER
        case createPlaylistRightBarAction:
            return NSLocalizedString(@"Create Playlist", nil);
            break;
        case createContactRightBarAction:
            return NSLocalizedString(@"Create Contact", nil);
            break;
        case createAlbumRightBarAction:
            return NSLocalizedString(@"Create Album", nil);
            break;
        case shareRightBarAction:
            return NSLocalizedString(@"Shares", nil);
            break;
        case lastnameRightBarAction:
            return NSLocalizedString(@"Sort-LastName", nil);
            break;
        case firstnameRightBarAction:
            return NSLocalizedString(@"Sort-FirstName", nil);
            break;
        case modifiedRightBarAction:
            return NSLocalizedString(@"Sort-Modified", nil);
            break;
        case createdRightBarAction:
            return NSLocalizedString(@"Sort-Created", nil);
            break;
        case favoriteRightBarAction:
            return NSLocalizedString(@"Sort-Favorite", nil);
            break;
        case idRightBarAction:
            return NSLocalizedString(@"Sort-ID", nil);
            break;
        case versionRightBarAction:
            return NSLocalizedString(@"Sort-Version", nil);
            break;
        case contactSearchRightBarAction:
            return NSLocalizedString(@"Search Contact", nil);
            break;
#endif
    case signOutRightBarAction:
            
            return NSLocalizedString(@"Sign Out", nil);
            break;
    case refreshAccountBarAction:
            return NSLocalizedString(@"refresh account", nil);
            break;
        default:
            return nil;
            break;
    }
}

- (void)didBeginDoingRightBarButtonAction:(rightBarAction)action {
    // abstract method, supposed to be executed by subclasses
}

- (void)didFinishDoingRightBarButtonAction:(rightBarAction)action :(id)result :(id)response {
    // abstract method, supposed to be executed by subclasses
}

- (id)dataSourceForRightBarButtonAction:(rightBarAction)action {
    return nil;
}

- (void)addRightBarButtonItemWithActions:(id) firstAction, ... {
    
    // first collect the actions y iterating through them
    // duplicate actions are ignored, hence chosen a NSMutableSet
    NSMutableArray *arguments = [[NSMutableArray alloc] init];
    id eachObject;
    va_list argumentList;
    if (firstAction)  {
        [arguments addObject:firstAction];
        va_start(argumentList, firstAction);
        while ((eachObject = va_arg(argumentList, id)))  {
            [arguments addObject: eachObject];
        }
        va_end(argumentList);
    }

    
    // create the bar button item using the actions
    ActionBarButtonItem *actionButton = [[ActionBarButtonItem alloc]
                                     initWithBarButtonSystemItem:UIBarButtonSystemItemAction
                                     target:self
                                     action:@selector(rightBarButtonClicked:)];
    actionButton.actions = arguments;
    
    self.navigationItem.rightBarButtonItem = actionButton;
}

- (void)executeAction:(rightBarAction)action silent:(BOOL)silent {
    switch (action) {
#ifdef KEPLER_FILE_BROWSER
        case emptyTrashRightBarAction:
            [self addEmptyTrashAction:nil];
            break;
            
        case createFolderRightBarAction:
            [self doCreateFolderActionCompletion:nil];
            break;
        case createVirtualFolderRightBarAction:
            [self doCreateVirtualFolderActionCompletion:nil];
            break;
        case photoMediaRightBarAction:
            [self getPhotoActionCompletion];
            break;
        case videoMediaRightBarAction:
            [self getVideoActionCompletion];
            break;
        case audioMediaRightBarAction:
            [self getAudioActionCompletion];
            break;
        case docMediaRightBarAction:
            [self getDocumentActionCompletion];
            break;
        case getFavoritesRightBarAction:
            [self doFavoritesActionCompletion];
            break;
        case videoFavoriteRightBarAction:
            [self getFavoritesActionCompletion:videoFavoriteResourceType];
            break;
        case audioFavoriteRightBarAction:
            [self getFavoritesActionCompletion:audioFavoriteResourceType];;
            break;
        case photoFavoriteRightBarAction:
            [self getFavoritesActionCompletion:photoFavoriteResourceType];
            break;
        case docFavoriteRightBarAction:
            [self getFavoritesActionCompletion:documentFavoriteResourceType];
            break;
        case folderFavoriteRightBarAction:
           [self getFavoritesActionCompletion:folderFavoriteResourceType];
            break;
        case allFavoriteRightBarAction:
           [self getFavoritesActionCompletion:allFavoriteResourceType];
            break;
        case fileFavoriteRightBarAction:
           [self getFavoritesActionCompletion:fileFavoriteResourceType];
            break;
            
#endif
        case accountInfoRightBarAction:
            [self doAccountInfoActionCompletion];
            break;
        case searchRightBarAction:
            [self doSearchActionCompletion];
            break;
        case signOutRightBarAction:
            [self doLogoutActionCompletion:silent];
            break;
        case refreshAccountBarAction:
            [self doRefreshAccountCompletion];
            break;
#ifdef HUBBLE_MEDIA_BROWSER
        case createPlaylistRightBarAction:
        case createAlbumRightBarAction:
            [self doCreatePlaylistAction:action completion:nil];
            break;
        case shareRightBarAction:
            [self doShareActionCompletion];
            break;
        case createContactRightBarAction:
            [self doCreateContactCompletion];
            break;
#endif
        default:
            break;
    }
}


#pragma mark -
- (void)rightBarButtonClicked:(id)sender {
    
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil
                                                                             message:nil
                                                                      preferredStyle:UIAlertControllerStyleActionSheet];

    ActionBarButtonItem *senderButton = (ActionBarButtonItem *)sender;
    
    for (NSNumber *actionObj in senderButton.actions) {
        switch ([actionObj intValue]) {
            case accountInfoRightBarAction:
                [self addAccountInfoAction:alertController];
                break;
            case searchRightBarAction:
                [self addSearchAction:alertController];
                break;
            case signOutRightBarAction:
                [self addSignOutAction:alertController];
                break;
            case refreshAccountBarAction:
                [self addRefreshAccountAction:alertController];
                break;
#ifdef KEPLER_FILE_BROWSER
            case emptyTrashRightBarAction:
                [self addEmptyTrashAction:alertController];
                break;
            case accessTokenRightBarAction:
                [self addAccessTokenAction:alertController];
                break;
            case createVirtualFolderRightBarAction:
                [self addCreateVirtualFolderAction:alertController];
                break;
            case createFolderRightBarAction:
                [self addCreateFolderAction:alertController];
                break;
            case uploadFileRightBarAction:
                [self addUploadFileAction:alertController];
                break;
            case photoMediaRightBarAction:
                [self addGetPhotoFileAction:alertController];
                break;
            case audioMediaRightBarAction:
                [self addGetAudioFileAction:alertController];
                break;
            case videoMediaRightBarAction:
                [self addGetVideoFileAction:alertController];
                break;
            case docMediaRightBarAction:
                [self addDocFileAction:alertController];
                break;
            case getFavoritesRightBarAction:
                [self addfavoritesAction:alertController];
                break;
            case videoFavoriteRightBarAction:
                [self addGetVideoFavoritesFileAction:alertController];
                break;
            case audioFavoriteRightBarAction:
                [self addGetAudioFavoritesFileAction:alertController];
                break;
            case photoFavoriteRightBarAction:
                [self addGetPhotoFavoritesFileAction:alertController];
                break;
            case docFavoriteRightBarAction:
                 [self addGetDocumentFavoritesFileAction:alertController];
                break;
            case folderFavoriteRightBarAction:
                 [self addFolderFavoritesFileAction:alertController];
                break;
            case fileFavoriteRightBarAction:
                 [self addFileFavoritesFileAction:alertController];
                break;
            case allFavoriteRightBarAction:
                 [self addAllFavoritesFileAction:alertController];
                break;
#endif
#ifdef HUBBLE_MEDIA_BROWSER
            case createPlaylistRightBarAction:
            case createAlbumRightBarAction:
                [self addCreatePlaylistAction:[actionObj intValue] alertController:alertController];
                break;
            case shareRightBarAction:
                [self addShareAction:alertController];
                break;
            case firstnameRightBarAction:
                [self addContactAction:alertController WithSortType:firstnameContactSortType withRightBarAction:firstnameRightBarAction];
                break;
            case lastnameRightBarAction:
                [self addContactAction:alertController WithSortType:lastnameContactSortType withRightBarAction:lastnameRightBarAction];
                break;
            case createdRightBarAction:
                [self addContactAction:alertController WithSortType:createdContactSortType withRightBarAction:createdRightBarAction];
                break;
            case modifiedRightBarAction:
                [self addContactAction:alertController WithSortType:modifiedContactSortType withRightBarAction:modifiedRightBarAction];
                break;
            case versionRightBarAction:
                [self addContactAction:alertController WithSortType:versionContactSortType withRightBarAction:versionRightBarAction];
                break;
            case idRightBarAction:
                [self addContactAction:alertController WithSortType:idContactSortType withRightBarAction:idRightBarAction];
                break;
            case favoriteRightBarAction:
                [self addContactAction:alertController WithSortType:favoriteContactSortType withRightBarAction:favoriteRightBarAction];
                break;
            case contactSearchRightBarAction:
                [self addContactSearchAction:alertController];
                break;
            case createContactRightBarAction:
                [self addCreateContactAction:alertController];
                break;
#endif
            default:
                break;
        }
    }

    [self addCancelAction:alertController];
    [self presentViewController:alertController animated:YES completion:nil];
    
}


#pragma mark -
#pragma mark Cancel

- (void)addCancelAction:(UIAlertController *)alertController {
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Cancel", nil)
                    style:UIAlertActionStyleCancel
                andAction:nil];
}

#ifdef KEPLER_FILE_BROWSER
#pragma mark -
#pragma mark Create Virtual Folder

- (void)doCreateVirtualFolderActionCompletion:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    
    alertController = [UIAlertController alertControllerWithTitle:NSLocalizedString(@"Enter Repo Name", nil)
                                                          message:nil
                                                   preferredStyle:UIAlertControllerStyleAlert];
    
    [self alertController:alertController showTextInputAlertWithTitle:nil andOKCompletionHandler:^(UIAlertAction *action) {
        UITextField *textField = [alertController.textFields firstObject];
        if (textField.text.length) {
            [blocksafeSelf didBeginDoingRightBarButtonAction:createVirtualFolderRightBarAction];
            
            /*
             @Description :
             This API is used to create a virtual folder. Once the create process is succeed/failed result will come with completion block.
             
             @Parameters :
             PCCreateVFolderAPIParams:
             * This class stores all the parameters required to create a virtual folder. It contains attributes such as folderName.
             * For providing access token to SDK user have to logged In.
             * folderName is required parameter to call this API.
             * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
             * More information about parameters you will find in PCCreateVFolderAPIParams.h class.
             
             @CompletionHandler :
             This completion block gets called once the create virtual folder process is succeed/failed. completion block contains below parameter.
             PCResource : This object stores metadata information of a virtual folder.
             PCAPIQueryResponse : Stores all the server response and error for this API.
             
             @To create a virtual folder, you must take the following steps:
             * Include VerizonCloudAPI.framework header file in your class.
             #import <VerizonCloudAPI/VerizonCloudAPI.h>
             * Initalize the PCCreateVFolderAPIParams class and provide the required parameter.
             * Initalize the PCAPISDK object and call the  createVirtualFolder:withCompletionHandler: API.
             * Perform the  createVirtualFolder:withCompletionHandler: API call using the code sample below.
             * Get the result (succeed or failed) in completion block.
             * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
             */
    
            // Intialize the PCCreateVFolderAPIParams object.
            PCCreateVFolderAPIParams *params = [[PCCreateVFolderAPIParams alloc] init];
            params.folderName = textField.text;
            
            // Initilize the PCAPISDK and call the API.
            [[PCAPISDK sharedInstance] createVirtualFolder:params withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response){
                // Get the result in this block.
                [blocksafeSelf didFinishDoingRightBarButtonAction:createVirtualFolderRightBarAction :metadata :response];
            }];
        }
    }];
}

- (void)addCreateVirtualFolderAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    __block UIAlertController *createVFolderAlertController;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Create Repo", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf doCreateVirtualFolderActionCompletion:createVFolderAlertController];
                }];
}

#pragma mark -
#pragma mark Create Folder

- (void)doCreateFolderActionCompletion:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;

    alertController = [UIAlertController alertControllerWithTitle:NSLocalizedString(@"Enter Folder Name", nil)
                                                          message:nil
                                                   preferredStyle:UIAlertControllerStyleAlert];

    [self alertController:alertController showTextInputAlertWithTitle:nil andOKCompletionHandler:^(UIAlertAction *action) {
        UITextField *textField = [alertController.textFields firstObject];
        if (textField.text.length) {
            [blocksafeSelf didBeginDoingRightBarButtonAction:createFolderRightBarAction];
            
            /*
             @Description :
             This API is used to create a folder. Once the create process is succeed/failed result will come with completion block.
             
             @Parameters :
             PCCreateFolderAPIParams:
             * This class stores all the parameters required to create a folder. It contains attributes such as path, name and overrideRule.
             * For providing access token to SDK user have to logged In.
             * path and name is required parameter to call this API.
             * overrideRule is optional parameter to call this API.
             * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
             * More information about parameters you will find in PCCreateFolderAPIParams.h class.
             
             @CompletionHandler :
             This completion block gets called once the create folder process is succeed/failed. completion block contains below parameter.
             PCResource : This object stores metadata information of a file or a folder.
             PCAPIQueryResponse : Stores all the server response and error for this API.
             
             @To create a folder, you must take the following steps:
             * Include VerizonCloudAPI.framework header file in your class.
             #import <VerizonCloudAPI/VerizonCloudAPI.h>
             * Initalize the PCCreateFolderAPIParams class and provide the required parameter.
             * Initalize the PCAPISDK object and call the  createFolder:withCompletionHandler: API.
             * Perform the  createFolder:withCompletionHandler: API call using the code sample below.
             * Get the result (succeed or failed) in completion block.
             * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
             */
            
            // Intialize the PCCreateFolderAPIParams object.
            PCCreateFolderAPIParams *createFolderParams = [[PCCreateFolderAPIParams alloc] init];
            createFolderParams.name = textField.text;
            createFolderParams.path = [blocksafeSelf dataSourceForRightBarButtonAction:createFolderRightBarAction];
        
            // Initilize the PCAPISDK and call the API.
            [[PCAPISDK sharedInstance] createFolder:createFolderParams withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {
                // Get the result in this block.
                [blocksafeSelf didFinishDoingRightBarButtonAction:createFolderRightBarAction :metadata :response];
            }];
        }
    }];
}

- (void)addCreateFolderAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    __block UIAlertController *createFolderAlertController;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Create Folder", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf doCreateFolderActionCompletion:createFolderAlertController];
                }];
}

#pragma mark -
#pragma mark Empty Trash
- (void)addEmptyTrashAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Empty Trash", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf emptyTrashAction];
                }];
}

- (void)emptyTrashAction {
    __weak UIViewController *blocksafeSelf = self;
    [self showOKCancelAlertWithTitle:NSLocalizedString(@"Empty Trash", nil) andMessage:NSLocalizedString(@"Do you wish to empty trash?", nil) okAction:^(UIAlertAction *action) {
        [blocksafeSelf didBeginDoingRightBarButtonAction:emptyTrashRightBarAction];
        
        /*
         @Description :
         This API is used to delete all the files and folder that are in a deleted state in the repository. Once the deletion process is succeed/failed result will come with completion block.
         
         @Parameters: nil
         * For providing access token to SDK user have to logged In.
         * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
         
         @CompletionHandler :
         This completion block gets called once deletion process is succeed/failed. completion block contains below parameter.
         PCAPIQueryResponse :
         * Stores all the server response and error for this API.
         
         @To delete all the files and folder that are in a deleted state in the repository, you must take the following steps:
         * Include VerizonCloudAPI.framework header file in your class.
            #import <VerizonCloudAPI/VerizonCloudAPI.h>
         * Initalize the PCAPISDK object and call the deleteTrashWithCompletionHandler: API.
         * Perform the deleteTrashWithCompletionHandler: API call using the code sample below.
         * Get the result (succeed or failed) in completion block.
         * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
         */
        
        // Sample code for deleteTrashWithCompletionHandler: API
        [[PCAPISDK sharedInstance] deleteTrashWithCompletionHandler:^(PCAPIQueryResponse *response) {
            [blocksafeSelf didFinishDoingRightBarButtonAction:emptyTrashRightBarAction :nil :response];
        }];
        
    } cancelAction:nil];
}

#pragma mark -
#pragma mark getPhotoList

- (void)getPhotoActionCompletion{
    
     __weak UIViewController *blocksafeSelf = self;
    [blocksafeSelf didBeginDoingRightBarButtonAction:photoMediaRightBarAction];
    
    // Sample code for getPhotosWithCompletionHandler: API
    
    [[PCAPISDK sharedInstance] getPhotosWithCompletionHandler:^(NSArray *itemList, PCAPIQueryResponse *response) {
        
       [blocksafeSelf didFinishDoingRightBarButtonAction:photoMediaRightBarAction :itemList :response];
        
    }];
}

- (void)addGetPhotoFileAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Photo", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf getPhotoActionCompletion];
                }];
}



#pragma mark -
#pragma mark getVideoList

- (void)getVideoActionCompletion{
    
    __weak UIViewController *blocksafeSelf = self;
    [blocksafeSelf didBeginDoingRightBarButtonAction:videoMediaRightBarAction];
    
    // Sample code for getVideosWithCompletionHandler: API
    
    [[PCAPISDK sharedInstance] getVideosWithCompletionHandler:^(NSArray *itemList, PCAPIQueryResponse *response) {
        
        [blocksafeSelf didFinishDoingRightBarButtonAction:videoMediaRightBarAction :itemList :response];
        
    }];
}

- (void)addGetVideoFileAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Video", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf getVideoActionCompletion];
                }];
}

#pragma mark -
#pragma mark getAudioList

- (void)getAudioActionCompletion{
    
    __weak UIViewController *blocksafeSelf = self;
    [blocksafeSelf didBeginDoingRightBarButtonAction:audioMediaRightBarAction];
    
    // Sample code for getAudiosWithCompletionHandler: API
    
    [[PCAPISDK sharedInstance] getAudiosWithCompletionHandler:^(NSArray *itemList, PCAPIQueryResponse *response) {
         [blocksafeSelf didFinishDoingRightBarButtonAction:audioMediaRightBarAction :itemList :response];
    }];
}

- (void)addGetAudioFileAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Audio", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf getAudioActionCompletion];
                }];
}

#pragma mark -
#pragma mark getDocumentList

- (void)getDocumentActionCompletion{
    __weak UIViewController *blocksafeSelf = self;
    [blocksafeSelf didBeginDoingRightBarButtonAction:docMediaRightBarAction];
   
    /*
     @Description :
     This API is used to fetch all documents from the account. Once the fetch process is succeed/failed result will come with completion block.
     
     @Parameters: nil
     * For providing access token to SDK user have to logged In.
     
     @CompletionHandler :
     This completion block gets called once fetch process is succeed/failed. completion block contains below parameter.
     PCAPIQueryResponse :
     * Stores all the server response and error for this API.
     NSArray :
     * An array of PCFile object. PCFile class stores the metadata information of file.
     * Using PCFile object we can perform all FOPS operations Such as move,copy,rename and delete.
     
     @To fetch all documents from the account, you must take the following steps:
     * Include VerizonCloudAPI.framework header file in your class.
     #import <VerizonCloudAPI/VerizonCloudAPI.h>
     * Initalize the PCAPISDK object and call the getDocumentsWithCompletionHandler: API.
     * Perform the getDocumentsWithCompletionHandler: API call using the code sample below.
     * Get the result (succeed or failed) in completion block.
     * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
     */
    
    // Sample code for getDocumentsWithCompletionHandler: API

    [[PCAPISDK sharedInstance] getDocumentsWithCompletionHandler:^(NSArray *itemList, PCAPIQueryResponse *response) {
        
         [blocksafeSelf didFinishDoingRightBarButtonAction:docMediaRightBarAction :itemList :response];
        
    }];
}

- (void)addDocFileAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Document", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf getDocumentActionCompletion];
                }];
}

#pragma mark -
#pragma mark Upload File


- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary *)info {
    
    [picker dismissViewControllerAnimated:YES completion:NULL];
    
    __block NSString *filePath = nil;
    
    
    NSURL *assetURL = [info valueForKey:UIImagePickerControllerReferenceURL];
    
    ALAssetsLibraryAssetForURLResultBlock resultblock = ^(ALAsset *assetItem) {
        
        NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
        NSString *documentsDirectory = [paths objectAtIndex:0];
        
        ALAssetRepresentation *representation = [assetItem defaultRepresentation];
        NSString *fileName = [representation filename];
        filePath = [documentsDirectory stringByAppendingPathComponent:fileName];
        
        
        if (info[UIImagePickerControllerMediaType] == (NSString*)kUTTypeImage) {
            
            NSData *imageData = UIImagePNGRepresentation(info[UIImagePickerControllerOriginalImage]);
            [imageData writeToFile:filePath atomically:YES];
            
        } else if (info[UIImagePickerControllerMediaType] == (NSString*)kUTTypeMovie) {
            
            NSURL *videoURL = info[UIImagePickerControllerMediaURL];
            filePath = [videoURL path];
        }
        
        NSString *folderPath = [self dataSourceForRightBarButtonAction:createFolderRightBarAction];
        
        UploadObj *obj = [[UploadObj alloc] init];
        obj.fileName = fileName;
        obj.destinationFolder = folderPath;
        obj.filePath = filePath;
        obj.thumbnailImage = [UIImage imageWithCGImage:[assetItem thumbnail]];
        
        [[UploadManager sharedInstance] addFileToUploadQueue:obj];

    };
    
    ALAssetsLibraryAccessFailureBlock failureblock  = ^(NSError *myerror) {
        DLog(@"Can not get asset - %@",[myerror localizedDescription]);
    };
    
    
    if (assetURL.absoluteString.length) {
        ALAssetsLibrary *assetsLibrary = [[ALAssetsLibrary alloc] init];
        [assetsLibrary assetForURL:assetURL
                       resultBlock:resultblock
                      failureBlock:failureblock];
    }
}

- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker {
    [picker dismissViewControllerAnimated:YES completion:NULL];
}



- (void)doUploadFileActionCompletion {
    UIImagePickerController *picker = [[UIImagePickerController alloc] init];
    picker.delegate = self;
    picker.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
    picker.mediaTypes = [[NSArray alloc] initWithObjects: (NSString *) kUTTypeMovie, (NSString*) kUTTypeImage, nil];
    [self presentViewController:picker animated:YES completion:NULL];
}

- (void)addUploadFileAction:(UIAlertController *)alertController {
    __weak UIViewController *blockSafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Upload File", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blockSafeSelf doUploadFileActionCompletion];
                }];
}


#pragma mark -
#pragma mark Favorites

- (void)doFavoritesActionCompletion {
    
    UIStoryboard *storyBoard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    FavoritesViewController *controller = (FavoritesViewController*)[storyBoard instantiateViewControllerWithIdentifier: @"FavoritesViewController"];
    [self.navigationController pushViewController:controller animated:YES];
    
}

- (void)addfavoritesAction:(UIAlertController *)alertController {
    __weak UIViewController *blockSafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Favorites", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    [blockSafeSelf doFavoritesActionCompletion];
                }];
}
#pragma mark -
#pragma mark get favorites of resource type

- (void)getFavoritesActionCompletion:(FavoriteResourceType) resourceType{
    
    __weak UIViewController *blocksafeSelf = self;
    [blocksafeSelf didBeginDoingRightBarButtonAction:photoFavoriteRightBarAction];
    
    /*
     @Description -
     This API is used to fetch the list of favorite files or folders from the user account. Once the fetch process is succeed/failed result will come with completion block.
     
     @Parameters -
     PCGetFavoritesAPIParams:
     * This class stores all the parameters required to fetch the list of favorite files or folders from the user account.
     *  For providing access token to SDK user have to logged In.
     * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
     * FavoriteResourceType is optional parameter to call this API.
     * More information about parameters you will find in PCGetFavoritesAPIParams.h class.
     
     @CompletionHandler :
     This completion block gets called once the fetch process is succeed/failed. completion block contains below parameter.
     PCAPIQueryResponse :
     * Stores all the server response and error for this API.
     NSArray :
     * An array of PCMetadata object.
     
     @To fetch fetch the list of favorite files or folders under the specified virtual folder, you must take the following steps:
     * Include VerizonCloudAPI.framework header file in your class.
     #import <VerizonCloudAPI/VerizonCloudAPI.h>
     * Initalize the PCGetFavoritesAPIParams class and provide the required parameter.
     * Initalize the PCAPISDK object and call the getFavorites:withCompletionHandler: API.
     * Perform the getFavorites:withCompletionHandler: API call using the code sample below.
     * Get the result (succeed or failed) in completion block.
     * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
     */

    
    PCGetFavoritesAPIParams *params = [[PCGetFavoritesAPIParams alloc] init];
    params.favResourceType = resourceType;
    
    [[PCAPISDK sharedInstance] getFavorites:params withCompletionHandler:^(NSArray *favoritesItems, PCAPIQueryResponse *response) {
         [blocksafeSelf didFinishDoingRightBarButtonAction:photoFavoriteRightBarAction :favoritesItems :response];
    }];
}

- (void)addGetPhotoFavoritesFileAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Image", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf getFavoritesActionCompletion:photoFavoriteResourceType];
                }];
}

- (void)addGetVideoFavoritesFileAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Video", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf getFavoritesActionCompletion:videoFavoriteResourceType];
                }];
}

- (void)addGetAudioFavoritesFileAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Music", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf getFavoritesActionCompletion:audioFavoriteResourceType];
                }];
}
- (void)addGetDocumentFavoritesFileAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Documents", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf getFavoritesActionCompletion:documentFavoriteResourceType];
                }];
}
- (void)addFileFavoritesFileAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"File", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf getFavoritesActionCompletion:fileFavoriteResourceType];
                }];
}
- (void)addFolderFavoritesFileAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Folder", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf getFavoritesActionCompletion:folderFavoriteResourceType];
                }];
}
- (void)addAllFavoritesFileAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"All", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf getFavoritesActionCompletion:allFavoriteResourceType];
                }];
}

#endif

#pragma mark -
#pragma mark Search

- (void)doSearchActionCompletion {

    UIStoryboard *storyBoard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    SearchConfigurationViewController *controller = (SearchConfigurationViewController*)[storyBoard instantiateViewControllerWithIdentifier: @"SearchConfigurationViewController"];
#ifdef HUBBLE_MEDIA_BROWSER
    controller.hidesBottomBarWhenPushed = YES;
#endif
    [self.navigationController pushViewController:controller animated:YES];

}

- (void)addSearchAction:(UIAlertController *)alertController {
    __weak UIViewController *blockSafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Search", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {

                    [blockSafeSelf doSearchActionCompletion];
                }];
}
#pragma mark -
#pragma mark acessToken

- (void)addAccessTokenAction:(UIAlertController *)alertController {
    
    __weak UIViewController *blockSafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Access Token", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blockSafeSelf doAccessTokenActionCompletion];
                }];
}

- (void)doAccessTokenActionCompletion {
    
    NSString *accessToken = [[PCAPISDK sharedInstance] fetchAccessToken];
    
    [self showOKSendAlertWithTitle:NSLocalizedString(@"Access Token", nil) andMessage:accessToken okAction:^(UIAlertAction *action) {
        if ([MFMailComposeViewController canSendMail])
        {
            MFMailComposeViewController *mail = [[MFMailComposeViewController alloc] init];
            mail.mailComposeDelegate = self;
            [mail setSubject:@"Access Token"];
            [mail setMessageBody:[NSString stringWithFormat:@"Access Token :\n%@",accessToken] isHTML:NO];
            [self presentViewController:mail animated:YES completion:NULL];
        }
        else
        {
            [self showEmailConfigurationAlert];
        }
    } cancelAction:nil];
    
}

#pragma mark -
#pragma mark mail delegate.
- (void)mailComposeController:(MFMailComposeViewController *)controller didFinishWithResult:(MFMailComposeResult)result error:(NSError *)error
{
    switch (result) {
        case MFMailComposeResultSent:
            DLog(@"You sent the email.");
            break;
        case MFMailComposeResultSaved:
            DLog(@"You saved a draft of this email.");
            break;
        case MFMailComposeResultCancelled:
            DLog(@"You cancelled sending this email.");
            break;
        case MFMailComposeResultFailed:
            DLog(@"Mail failed: An error occurred when trying to compose this email.");
            break;
        default:
            DLog(@"An error occurred when trying to compose this email.");
            break;
    }
    
    [self dismissViewControllerAnimated:YES completion:NULL];
}
#pragma mark -
#pragma mark AccountInfo

- (void)fetchAccountInfo {
    __weak UIViewController *blockSafeSelf = self;
    
    /*
     @Description :
     This API is used to fetch account details. Once the fetch process is succeed/failed result will come with completion block.
     
     @Parameters : nil
     * For providing access token to SDK user have to logged In.

     @CompletionHandler :
     This completion block gets called once the fetch process is succeed/failed. completion block contains below parameter.
      PCAccountInfo : This object stores quota (Total quota space available to the user on the cloud in bytes) ,quotaUsed(Total quota space used in bytes) , ID (The user identifier) and alias (Username. Available only for MDN accounts).
     PCAPIQueryResponse : Stores all the server response and error for this API.
     
     @To fetch account details, you must take the following steps:
     * Include VerizonCloudAPI.framework header file in your class.
     #import <VerizonCloudAPI/VerizonCloudAPI.h>
     * Initalize the PCAPISDK object and call the  fetchAccountInfoWithCompletionHandler: API.
     * Perform the  fetchAccountInfoWithCompletionHandler: API call using the code sample below.
     * Get the result (succeed or failed) in completion block.
     * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
     */
    
    // Initilize the PCAPISDK and call the API.
    

    [[PCAPISDK sharedInstance] fetchAccountInfoWithCompletionHandler:^(PCAccountInfo *accountInfo, PCAPIQueryResponse *response) {
        
        [blockSafeSelf didFinishDoingRightBarButtonAction:accountInfoRightBarAction :accountInfo :response];
        
    }];
}

- (void)doAccountInfoActionCompletion {
    [self didBeginDoingRightBarButtonAction:accountInfoRightBarAction];
    [self fetchAccountInfo];
}

- (void)addAccountInfoAction:(UIAlertController *)alertController {
    __weak UIViewController *blockSafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Account Info", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blockSafeSelf doAccountInfoActionCompletion];
                }];
}

#pragma mark -
#pragma mark Refresh Account

- (void)refreshAccount {
    __weak UIViewController *blocksafeSelf = self;
    
    /*
     @Description -
     This API is used to refresh current session. Once session is refreshed PCAuthenticationResponse object returned with updated value. Result will come with completion block. For using this API user required to loggged in.
     
     @ReturnType - PCAPI object Using this return type we can cancel the ongoing request.
     
     @CompletionHandler -
     This completion block gets called once refresh current session process is done. completion block contains below parameters.
     PCAuthenticationResponse : This object stores access token and related credentials information. If user is successfully refreshed the session, this object update stored value in keychain and this will remain in keychain until user forcefully calls logout.
     PCAPIQueryResponse : Stores all the server response and error for this API.
     
     @To refresh current session, you must take the following steps:
      * Initialize PCAPISDK and call the refreshSessionWithCompletionHandler: API.
      * Get the result in completion block.
     */
    
    [[PCAPISDK sharedInstance] refreshSessionWithCompletionHandler:^(PCAuthenticationResponse *authenticationCredentials, PCAPIQueryResponse *response) {
        [blocksafeSelf didFinishDoingRightBarButtonAction:refreshAccountBarAction :authenticationCredentials :response];
    }];
}

- (void)doRefreshAccountCompletion {

    [self didBeginDoingRightBarButtonAction:refreshAccountBarAction];
    [self refreshAccount];
}

- (void)addRefreshAccountAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Refresh Account", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf doRefreshAccountCompletion];
                }];
}

#pragma mark -
#pragma mark SignOut

- (void)logOutUser {
    /*
     @Description-
     This method is to logout user from current session. Access token stored in keychain is also removed. Returns YES if token was successfully removed from keychain. If user is not logged in , it will return NO.
     @Parameters - nil
     @ReturnType - BOOL.
     */
    
    [[PCAPISDK sharedInstance] logout];
#ifdef KEPLER_FILE_BROWSER
    [[UploadManager sharedInstance] cleanUpload];
#endif
    [self dismissSelfRecursivelyAnimated:YES];
}

- (void)doLogoutActionCompletion:(BOOL)silent {
   
    __weak UIViewController *blocksafeSelf = self;
    if (silent) {
        /*
         @Description-
         This method is used to check availability of access token. Returns YES if user has an access token.
         @Parameters - nil
         @ReturnType - BOOL.
         */
        if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
            [blocksafeSelf logOutUser];
        }
    } else {
        [self showOKCancelAlertWithTitle:NSLocalizedString(@"Sign Out", nil) andMessage:NSLocalizedString(@"Do you wish to continue Sign Out?", nil) okAction:^(UIAlertAction *action) {
            if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
                [blocksafeSelf logOutUser];
            }
        } cancelAction:nil];
    }
}


- (void)addSignOutAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Sign Out", nil)
                    style:UIAlertActionStyleDestructive
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf doLogoutActionCompletion:NO];
                }];
}

#ifdef HUBBLE_MEDIA_BROWSER

#pragma mark -
#pragma mark Contact sort

- (void)addContactAction:(UIAlertController *)alertController
            WithSortType:(ContactSortType)sortType
      withRightBarAction:(rightBarAction)rightBarAction {
    
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString([self stringForRightBarAction:rightBarAction], nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf getContactsActionCompletion:rightBarAction WithSortType:sortType];
                }];
}

//get Contact based on sorting.

- (void)getContactsActionCompletion:(rightBarAction)rightBarAction
                       WithSortType:(ContactSortType)sortType{
    
    __weak UIViewController *blocksafeSelf = self;
    [blocksafeSelf didBeginDoingRightBarButtonAction:rightBarAction];
    
    /*
     @Description -
     This API is used to fetch contact details in an account. Once the fetch process is succeed/failed result will come with completion block.
     
     @Parameters -
     PCContactAPIParams:
     * This class stores all the parameters required to fetch contacts from Verizon Cloud account.
     *  For providing access token to SDK user have to logged In.
     * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
     * page, count, sort and query are optional parameter to call this API.
     * use setSortKey:isAscending: method to set value for sort.
     * use setQueryValue:forField: method to set value for query.
     * More information about parameters you will find in PCContactAPIParams.h class.
     
     
     @CompletionHandler :
     This completion block gets called once contact fetch process is succeed/failed. completion block contains below parameter.
     PCAPIQueryResponse :
     * Stores all the server response and error for this API.
     NSArray :
     * An array of PCContact object.
     
     
     @To fetch all contact details in an account, you must take the following steps:
     * Include VerizonCloudAPI.framework header file in your class.
     #import <VerizonCloudAPI/VerizonCloudAPI.h>
     * Initalize the PCAPISDK object and call the fetchContacts:WithCompletionHandler: API.
     * Perform the fetchContacts:WithCompletionHandler: API call using the code sample below.
     * Get the result (succeed or failed) in completion block.
     * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
     */
    
    // Sample code for fetchContacts:WithCompletionHandler: API
    
    PCContactAPIParams *params = [[PCContactAPIParams alloc] init];
    [params setSortKey:sortType isAscending:YES];
    
    [[PCAPISDK sharedInstance] fetchContacts:params withCompletionHandler:^(NSArray *contacts, PCAPIQueryResponse *response) {
         [blocksafeSelf didFinishDoingRightBarButtonAction:rightBarAction :contacts :response];
    }];
}

#pragma mark -
#pragma mark contact search

- (void)doContactSearchActionCompletion:(UIAlertController *)alertController {
    
    __weak UIViewController *blocksafeSelf = self;
    
    UIAlertController * alert=   [UIAlertController
                                  alertControllerWithTitle:@"Contact Search"
                                  message:@"Enter field and value"
                                  preferredStyle:UIAlertControllerStyleAlert];
    
    UIAlertAction* ok = [UIAlertAction actionWithTitle:@"Search" style:UIAlertActionStyleDefault
                                               handler:^(UIAlertAction * action) {
                                                   //Do Some action here
                                                   NSString *field = [alert.textFields.firstObject.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
                                                   NSString *value = [alert.textFields.lastObject.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
                                                   
                                                   
                                                    if (field.length && value.length) {
                                                        
                                                       
                                                        [blocksafeSelf didBeginDoingRightBarButtonAction:contactSearchRightBarAction];
                                                        
                                                        /*
                                                         @Description -
                                                         This API is used to fetch contact details in an account. Once the fetch process is succeed/failed result will come with completion block.
                                                         
                                                         @Parameters -
                                                         PCContactAPIParams:
                                                         * This class stores all the parameters required to fetch contacts from Verizon Cloud account.
                                                         *  For providing access token to SDK user have to logged In.
                                                         * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
                                                         * page, count, sort and query are optional parameter to call this API.
                                                         * use setSortKey:isAscending: method to set value for sort.
                                                         * use setQueryValue:forField: method to set value for query.
                                                         * More information about parameters you will find in PCContactAPIParams.h class.
                                                         
                                                         
                                                         @CompletionHandler :
                                                         This completion block gets called once contact fetch process is succeed/failed. completion block contains below parameter.
                                                         PCAPIQueryResponse :
                                                         * Stores all the server response and error for this API.
                                                         NSArray :
                                                         * An array of PCContact object.
                                                         
                                                         
                                                         @To fetch all contact details in an account, you must take the following steps:
                                                         * Include VerizonCloudAPI.framework header file in your class.
                                                         #import <VerizonCloudAPI/VerizonCloudAPI.h>
                                                         * Initalize the PCAPISDK object and call the fetchContacts:WithCompletionHandler: API.
                                                         * Perform the fetchContacts:WithCompletionHandler: API call using the code sample below.
                                                         * Get the result (succeed or failed) in completion block.
                                                         * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
                                                         */
                                                        
                                                        // Sample code for fetchContacts:WithCompletionHandler: API
                                                        
                                                        PCContactAPIParams *params = [[PCContactAPIParams alloc] init];
                                                        [params setQueryValue:value forField:field];
                                                        
                                                        [[PCAPISDK sharedInstance] fetchContacts:params withCompletionHandler:^(NSArray *contacts, PCAPIQueryResponse *response) {
                                                            [blocksafeSelf didFinishDoingRightBarButtonAction:contactSearchRightBarAction :contacts :response];
                                                        }];
                                                        
                                                    } else {
                                                         [self showOKAlertWithTitle:NSLocalizedString(@"Empty", nil) andMessage:NSLocalizedString(@"Please enter field or value", nil) attributedString:nil okAction:nil];
                                                    }
                                               }];
    UIAlertAction* cancel = [UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleDefault
                                                   handler:^(UIAlertAction * action) {
                                                       [alert dismissViewControllerAnimated:YES completion:nil];
                                                   }];
    
    [alert addAction:ok];
    [alert addAction:cancel];
    
    [alert addTextFieldWithConfigurationHandler:^(UITextField *textField) {
        textField.placeholder = @"Field";
    }];
    [alert addTextFieldWithConfigurationHandler:^(UITextField *textField) {
        textField.placeholder = @"Value";
    }];
    
    [self presentViewController:alert animated:YES completion:nil];
}

- (void)addContactSearchAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    __block UIAlertController *contactSearchAlertController;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Search Contact", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf doContactSearchActionCompletion:contactSearchAlertController];
                }];
}

#pragma mark -
#pragma mark Create Contact

- (void)doCreateContactCompletion {
    CreateContactViewController *createContactViewController = [CreateContactViewController viewControllerForNewContact:nil];
    createContactViewController.allowsEditing = YES;
    createContactViewController.delegate = createContactViewController;
    createContactViewController.mediaViewController = (MediaGroupViewController *)self;
    [self.navigationController pushViewController:createContactViewController animated:YES];
    
}

- (void)addCreateContactAction:(UIAlertController *)alertController {
     __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Create Contact", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf doCreateContactCompletion];
                }];
}

#pragma mark -
#pragma mark Playlist

- (void)doCreatePlaylistAction:(rightBarAction)buttonAction completion:(UIAlertController *)alertController {

    __weak UIViewController *blocksafeSelf = self;
 
    
    alertController = [UIAlertController alertControllerWithTitle:[self stringForRightBarAction:buttonAction]
                                                          message:nil
                                                   preferredStyle:UIAlertControllerStyleAlert];
    
    [self alertController:alertController showTextInputAlertWithTitle:nil andOKCompletionHandler:^(UIAlertAction *action) {
        UITextField *textField = [alertController.textFields firstObject];
        if (textField.text.length) {
            [blocksafeSelf didBeginDoingRightBarButtonAction:createPlaylistRightBarAction];
            // Intialize the PCCreateplaylistAPIParams object.

            PCCreateplaylistAPIParams *createPlaylistAPIParams = [[PCCreateplaylistAPIParams alloc] init];
            createPlaylistAPIParams.name = textField.text;
            createPlaylistAPIParams.type = [blocksafeSelf dataSourceForRightBarButtonAction:createPlaylistRightBarAction];
            
            /*
             @Description :
             This API is used to create a playlist. Once the create process is succeed/failed result will come with completion block.
             
             @Parameters :
             PCCreateplaylistAPIParams:
             * This class stores all the parameters required to create a playlist. It contains attributes such as name, type and paths.
             * For providing access token to SDK user have to logged In.
             * name and type are required parameter to call this API.
             * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
             * More information about parameters you will find in PCCreateplaylistAPIParams.h class.
             
             @CompletionHandler :
             This completion block gets called once the create process is succeed/failed. completion block contains below parameter.
             PCAPIQueryResponse : Stores all the server response and error for this API.
             
             @To create a playlist, you must take the following steps:
             * Include VerizonCloudAPI.framework header file in your class.
             #import <VerizonCloudAPI/VerizonCloudAPI.h>
             * Initalize the PCCreateplaylistAPIParams class and provide the required parameter.
             * Initalize the PCAPISDK object and call the  createPlaylist:withCompletionHandler: API.
             * Perform the  createPlaylist:withCompletionHandler: API call using the code sample below.
             * Get the result (succeed or failed) in completion block.
             * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
             */

            // Initilize the PCAPISDK and call the API.
            
            [[PCAPISDK sharedInstance] createPlaylist:createPlaylistAPIParams withCompletionHandler:^(PCPlaylist *playlist, PCAPIQueryResponse *response) {
                // Get the result in this block.
                [blocksafeSelf didFinishDoingRightBarButtonAction:createPlaylistRightBarAction :playlist :response];
            }];
        }
    }];
}

- (void)addCreatePlaylistAction:(rightBarAction)buttonAction alertController:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    
    __block UIAlertController *createlaylistAlertController = nil;

    [self alertController:alertController
       addActionWithTitle:[self stringForRightBarAction:buttonAction]
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf doCreatePlaylistAction:buttonAction completion:createlaylistAlertController];
                }];
}

#pragma mark Shares

- (void)addShareAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Shares", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf doShareActionCompletion];
                }];
}
- (void)doShareActionCompletion {
    
    UIStoryboard *storyBoard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    SharesTableViewController *controller = (SharesTableViewController*)[storyBoard instantiateViewControllerWithIdentifier: @"SharesTableViewController"];
    [self.navigationController pushViewController:controller animated:YES];
    
    
}

#endif


@end
