//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"

@interface PCFetchMetaDataAPITests : XCTestCase

@end

@implementation PCFetchMetaDataAPITests

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}


/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil.
 */
- (void)testNilAuthToken {
    // If User is not logged in, fetch meta data should throw an exception.
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCMetaDataAPIParams *params = [[PCMetaDataAPIParams alloc] init];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] fetchMetadata:params withCompletionHandler:^(PCMetadata *metadata, PCAPIQueryResponse *response) {}],@"Fetch metadata with nil access token");
    }
}

/*!
 * Test Case: path is a required parameter. SDK should throw an exception in case path is nil.
 */
- (void) testRequiredPathForFetchMetadata {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        PCMetaDataAPIParams *params = [[PCMetaDataAPIParams alloc] init];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] fetchMetadata:params withCompletionHandler:^(PCMetadata *metadata, PCAPIQueryResponse *response) {}] ,@"Fetch metadata without required path");
        
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}

/*!
 * Test Case:
 * Call metdata API and fetch all file and folders.
 * delete the file from the list.
 * Again call metadata API and check same file displaying on the list.
 * Deleted file delete value change to true.
 */
-(void) testWhenIncludeDeletedSelectionTrue {
    
    //if true, response will include deleted files and folders.
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCMetaDataAPIParams *params = [[PCMetaDataAPIParams alloc] init];
        params.path = @"/VZMOBILE";
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Find deleted file in metadata response"];
        
        [[PCAPISDK sharedInstance] fetchMetadata:params withCompletionHandler:^(PCMetadata *metadata, PCAPIQueryResponse *response) {
          
            switch (response.urlResponse.statusCode) {
                    
                case PCAPIResponseCodeOK: {
                    
                    if (metadata.subFiles.count) {
                        PCResource *resource = [metadata.subFiles objectAtIndex:0];
                        
                        PCDeleteParams *params = [[PCDeleteParams alloc] init];
                        params.path = resource.fullPath;
                        
                        [[PCAPISDK sharedInstance] deletePath:params withCompletionHandler:^(PCAPIQueryResponse *response) {
                            
                            switch (response.urlResponse.statusCode) {
                                case PCAPIResponseCodeOK: {
                                    PCMetaDataAPIParams *params = [[PCMetaDataAPIParams alloc] init];
                                    params.path = @"/VZMOBILE";
                                    params.includeDeleted = YES;
                                    
                                    [[PCAPISDK sharedInstance] fetchMetadata:params withCompletionHandler:^(PCMetadata *metadata, PCAPIQueryResponse *response) {
                                        switch (response.urlResponse.statusCode) {
                                            case PCAPIResponseCodeOK: {
                                                BOOL isContain  = NO;
                                                for (PCResource *deletedResource in metadata.subFiles) {
                                                    if ([deletedResource.fullPath isEqualToString:resource.fullPath]) {
                                                       XCTAssertTrue([deletedResource deleted] ,@"Deleted file should present in metadata response when user selected true in include deleted");
                                                        isContain = YES;
                                                    }
                                                }
                                                if (!isContain) {
                                                     XCTFail(@"Test could not be completed because include deleted scenario failed");
                                                }
                                            }
                                               
                                                break;
                                            default:
                                                 XCTFail(@"Test could not be completed because metadata API failed");
                                                break;
                                        }
                                        [expectation fulfill];
                                    }];
                                }
                                break;
                                default:
                                    XCTFail(@"Test could not be completed because delete API failed");
                                    break;
                            }
                        }];
                    } else {
                        XCTFail(@"Test could not be completed because their is no file");
                    }
                    
                }
                break;
                    
                default:
                    XCTFail(@"Test could not be completed because metadata API failed");
                    break;
            }
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*3 handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        
        XCTFail(@"Test Failed due to nil access token");
        
    }
    
}

/*!
 * Test Case:
 * Call metdata API and fetch response.
 * Check count is more than 1
 * Check response only contains file.
 */
- (void) testMetadataResponseFilterOnlyFile {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        PCMetaDataAPIParams *params = [[PCMetaDataAPIParams alloc] init];
        params.path = @"/VZMOBILE";
        params.filter = @"file";
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Metadata response only contains files"];
        
        [[PCAPISDK sharedInstance] fetchMetadata:params withCompletionHandler:^(PCMetadata *metadata, PCAPIQueryResponse *response) {
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeOK: {
                    XCTAssertGreaterThan([metadata.count integerValue], 0, @"Test failed as there is no file or folder");
                    XCTAssertGreaterThan([metadata.subFiles count], 0 ,@"Test failed if metadata returning count it should some value");
                    XCTAssertEqual([metadata.subFolders count], 0 , @"Test failed response never return folder count");
                }
                    break;
                    
                default:
                    XCTFail(@"Test could not be completed because metadata API failed");
                    break;
            }
            [expectation fulfill];
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
    } else {
         XCTFail(@"Test Failed due to nil access token");
    }
    
}

/*!
 * Test Case:
 * Call metdata API and fetch response.
 * Check count is more than 1
 * Check response only contains folder.
 */
- (void) testMetadataResponseFilterOnlyFolder {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        PCMetaDataAPIParams *params = [[PCMetaDataAPIParams alloc] init];
        params.path = @"/VZMOBILE";
        params.filter = @"folder";
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Metadata response only contains folder"];
        
        [[PCAPISDK sharedInstance] fetchMetadata:params withCompletionHandler:^(PCMetadata *metadata, PCAPIQueryResponse *response) {
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeOK: {
                    XCTAssertGreaterThan([metadata.count integerValue], 0, @"Test failed as there is no file or folder");
                    XCTAssertGreaterThan([metadata.subFolders count], 0 ,@"Test failed if metadata returning count it should some value");
                    XCTAssertEqual([metadata.subFiles count], 0 , @"Test failed response never return file count");
                }
                    break;
                    
                default:
                    XCTFail(@"Test could not be completed because metadata API failed");
                    break;
            }
            [expectation fulfill];
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}

/*!
 * Test Case:
 * Configure PCMetaDataAPIParams object with sort query.
 * Call metdata API and fetch response.
 * Check response displaying based on sorting query.
 */

- (void) testSortingForMetaDataAPI {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCMetaDataAPIParams *params = [[PCMetaDataAPIParams alloc] init];
        params.path = @"/VZMOBILE";
        params.sort = @"size+asc";
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Metadata sort response based on sorting query"];
        
        [[PCAPISDK sharedInstance] fetchMetadata:params withCompletionHandler:^(PCMetadata *metadata, PCAPIQueryResponse *response) {
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeOK: {
                    PCMetadata *resource1 = [metadata.subFiles objectAtIndex:0];
                    PCMetadata *resource2 = [metadata.subFiles objectAtIndex:[metadata.subFiles count]-1];
                    XCTAssertGreaterThanOrEqual(resource2, resource1, @"Metadata API not sorting based on size");
                }
                    break;
                default:
                    XCTFail(@"Test could not be completed because metadata API failed");
                    break;
            }
            [expectation fulfill];
        }];
        
        
        [self waitForExpectationsWithTimeout:ExpectationTimeOut handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}

/*!
 * Test Case: If response code is 200, Metadata array contains zero or greater than zero value. Also PCMetadata object has to be of class PCMetadata type. In all other cases (error), Metadata has to be returned nil
 */
- (void)testToFetchMetaDataAPI {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Fetch metadata API Result"];
        
        PCMetaDataAPIParams *params = [[PCMetaDataAPIParams alloc] init];
        params.path = @"/VZMOBILE";
        
        [[PCAPISDK sharedInstance] fetchMetadata:params withCompletionHandler:^(PCMetadata *metadata, PCAPIQueryResponse *response) {
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeOK:
                    XCTAssertGreaterThanOrEqual([metadata.count integerValue], 0);
                    break;
                    
                default:
                     XCTAssertNil(metadata,@"Metatdata return nil");
                    break;
            }
            [expectation fulfill];
        }];
        
        [self waitForExpectationsWithTimeout:ExpectationTimeOut handler:^(NSError *error) {
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        
        XCTFail(@"Test Failed due to nil access token");
        
    }
    
}

@end
