//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.


#import <Foundation/Foundation.h>

/*!
 * Base class for all HTTP queries. 
 */
@interface PCAPI : NSObject

/*!
 * Returns the ID of NSURLSession associated with this api. nil if it's not a background session.
 */
@property (nonatomic, copy, readonly) NSString *sessionID;

/*!
 * Returns a unique ID of the api, which is actually the task identifier of the api.
 */
@property (nonatomic, assign, readonly) NSUInteger apiID;

/*!
 * Type of http request (GET, POST or PUT). By default GET.
 */
@property (nonatomic, copy, readonly) NSString *requestMethod; // GET, POST or PUT

/*!
 * Complete request url string.
 */
@property (nonatomic, copy, readonly) NSString *requestURLString;

/*!
 * Complete request url.
 */
@property (nonatomic, strong, readonly) NSURL *requestURL;

/*!
 * Request headers.
 */
@property (nonatomic, strong, readonly) NSDictionary *requestHeaders;

/*!
 * Request body.
 */
@property (nonatomic, strong, readonly) NSData *requestBody;

/*!
 * Base server url.
 */
@property (nonatomic, copy,readonly) NSString *serverBaseURL;

/*!
 * Call this method to cancel the ongoing webservice operation.
 */
- (void)cancelAPI;

FOUNDATION_EXPORT NSString * const PCAPIStartNotification;
FOUNDATION_EXPORT NSString * const PCAPICompleteNotification;
FOUNDATION_EXPORT NSString * const PCAPIInfoMessageLogNotification;
FOUNDATION_EXPORT NSString * const PCAPIQueryResponseKey;

@end
