//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.


#import <Foundation/Foundation.h>

typedef enum {
    
    PCAPIResponseCodeOK                     = 200,  // Success
    PCAPIResponseCodeResourceCreated        = 201,  // File/Folder/Playlist created
    PCAPIResponseCodeNoContent              = 204,  // No Content
    PCAPIResponseCodeBadRequest             = 400,  // [Bad Request] Query parameters are missing, or invalid.
    PCAPIResponseCodeUnauthorized           = 401,  // [Unauthorized] client_id is not valid or Bearer token is missing, expired, or invalid
    PCAPIResponseCodeResourceNotFound       = 404,  // [Not Found] Resource was not found.
    PCAPIResponseCodeResourceRemoved        = 410,  // [Gone] Resource was removed.
    PCAPIResponseCodeConfict                = 409,  // Conflict
    PCAPIResponseCodeLengthRequired         = 411,  // [Length Required] An entity was included in the request but a Content-Length header was not.
    PCAPIResponseCodePreconditionFailed     = 412,  // [Precondition Failed] An X-Header-ETag header was included in the request, and there are no changes to report.
    PCAPIResponseCodeUnsupportedMediaType   = 415,  // [Unsupported Media Type] Thumbnails are not supported for this type of file.
    PCAPIResponseCodeServiceUnavailable     = 503,  // [Service Unavailable] See response body for more detail.
    
} PCAPIResponseCode;

/*!
 * All PCAPIQuery responses are embedded in this format.
 */
@interface PCAPIQueryResponse : NSObject

/*!
 * Stores HTTP response from server.
 */
@property (nonatomic, readonly) NSHTTPURLResponse *urlResponse;

/*!
 * Stores temporary url of data downloaded through downloadDataAsynWithCompletionHandler: method in PCAPIQuery.
 */
@property (nonatomic, readonly) NSURL *downloadedURL;

/*!
 * Stores response of sendAsyncRequestWithCompletionHandler: method in PCAPIQuery.
 */
@property (nonatomic, readonly) NSDictionary *jsonResponse;

/*!
 * Stores http error details.
 */
@property (nonatomic, readonly) NSError *error;


@end
