//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.


#import <Foundation/Foundation.h>

/*!
 * The PCClientCredentials class stores client credentials like clientID, clientSecret and redirectURL received while registering an app with Verizon Developer Program.
 */

@interface PCClientCredentials : NSObject

/*!
 * Unique client ID received while registering third party application with Verizon Developer Program.
 */
@property (nonatomic, readonly) NSString *clientID;

/*!
 * A client secret received while registering third party application with Verizon Developer Program.
 */
@property (nonatomic, readonly) NSString *clientSecret;

/*!
 * Callback url mentioned by developer while registering third party application with Verizon Developer Program.
 */
@property (nonatomic, readonly) NSString *redirectURLString;

/*!
 * Set a value to keychainID, if auth token needs to be persisted and stored in Keychain.
 */
@property (nonatomic, readonly) NSString *keychainID;

/*!
 * Create a PCClientCredentials object passing clientID, clientSecret, redirectURL and keychainID. If keychainID is passed nil, accesstoken received through OAuth2 call is not saved in keychain.
 * @see https://developer.apple.com/library/ios/documentation/Security/Conceptual/keychainServConcepts/01introduction/introduction.html
 */
- (id)initWithClientID:(NSString *)clientID
          clientSecret:(NSString *)clientSecret
           redirectURL:(NSString *)redirectURL
            keychainID:(NSString *)keychainID;


@end
