//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import "PCResourceAPIParams.h"

/*!
 * This class stores base parameters for all file/folder operations like copy, rename and move.
 */
@interface PCFOPSAPIParams : PCResourceAPIParams

/*!
 * Full path of the file/folder for copy/move/rename operations (required).
 */
@property (nonatomic, copy) NSString *source;

/*!
 * Full path of the target file/folder for copy/move/rename operations. In case of files the full/changed name should be included in the target (required).
 */
@property (nonatomic, copy) NSString *target;

/*!
 * If 'true', ensures that name conflicts are resolved according to the value of the 'conflictsolve' parameter. Defaults to 'false', in which case the operation will overwrite an existing file/folder of the same name.
 */
@property (nonatomic, copy) NSString *safe;

/*!
 * Defines how name conflicts are resolved, if the 'safe' parameter is set to 'true'. If set to 'copy', a new file is written to the destination folder with a non-conflicting name. If not set, the operation is rolled back and operation returns '409 Conflict.
 */
@property (nonatomic, copy) NSString *conflictsolve;

/*!
 * Specifies what happens if a file of the same name exists at the target path and the 'safe' parameter is not set. Set to 'overwrite' to treat the copied file as completely new. Set to 'modify' to treat the new file as a modification of the old one. If the 'override' parameter is not set, the following algorithm is used: if the existing file has a 'deleted' attribute set to 'true', the file is overwritten; if the old file's 'deleted' attribute is false or not set, the file is modified.
 */
@property (nonatomic, copy) NSString *overrideRule;

@end
